﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Xenon.KifuNarabe.L01_Communication
{

    /// <summary>
    /// ************************************************************************************************************************
    /// あるデータを、別のデータに変換します。
    /// ************************************************************************************************************************
    /// </summary>
    public abstract class Converter01
    {

        #region プロパティ類
        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// アラビア数字。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public static string[] ARABIA_SUJI = new string[] { "１", "２", "３", "４", "５", "６", "７", "８", "９" };

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 漢数字。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public static string[] KAN_SUJI = new string[] { "一", "二", "三", "四", "五", "六", "七", "八", "九" };

        #endregion


        /// <summary>
        /// ************************************************************************************************************************
        /// 数値を漢数字に変換します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="num"></param>
        /// <returns></returns>
        public static string IntToKanSuji(int num)
        {
            string numStr;

            if (1 <= num && num <= 9)
            {
                numStr = KAN_SUJI[num - 1];
            }
            else
            {
                numStr = "×";
            }

            return numStr;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 数値をアラビア数字に変換します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="num"></param>
        /// <returns></returns>
        public static string IntToArabiaSuji(int num)
        {
            string numStr;

            if (1 <= num && num <= 9)
            {
                numStr = ARABIA_SUJI[num - 1];
            }
            else
            {
                numStr = "×";
            }

            return numStr;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// a～i を、1～9 に変換します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="?"></param>
        /// <returns></returns>
        public static int AlphabetToInt(string alphabet)
        {
            int num;

            switch (alphabet)
            {
                case "a":
                    num = 1;
                    break;
                case "b":
                    num = 2;
                    break;
                case "c":
                    num = 3;
                    break;
                case "d":
                    num = 4;
                    break;
                case "e":
                    num = 5;
                    break;
                case "f":
                    num = 6;
                    break;
                case "g":
                    num = 7;
                    break;
                case "h":
                    num = 8;
                    break;
                case "i":
                    num = 9;
                    break;
                default:
                    num = -1;
                    break;
            }

            return num;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 1～9 を、a～i に変換します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="?"></param>
        /// <returns></returns>
        public static string IntToAlphabet(int suji)
        {
            string alphabet;

            switch (suji)
            {
                case 1:
                    alphabet = "a";
                    break;
                case 2:
                    alphabet = "b";
                    break;
                case 3:
                    alphabet = "c";
                    break;
                case 4:
                    alphabet = "d";
                    break;
                case 5:
                    alphabet = "e";
                    break;
                case 6:
                    alphabet = "f";
                    break;
                case 7:
                    alphabet = "g";
                    break;
                case 8:
                    alphabet = "h";
                    break;
                case 9:
                    alphabet = "i";
                    break;
                default:
                    alphabet = "×";
                    break;
            }

            return alphabet;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// アラビア数字（全角半角）、漢数字を、int型に変換します。変換できなかった場合、0です。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="suji"></param>
        /// <returns></returns>
        public static int SujiToInt(string suji)
        {
            int result;

            switch (suji)
            {
                case "1":
                case "１":
                case "一":
                    result = 1;
                    break;

                case "2":
                case "２":
                case "二":
                    result = 2;
                    break;

                case "3":
                case "３":
                case "三":
                    result = 3;
                    break;

                case "4":
                case "４":
                case "四":
                    result = 4;
                    break;

                case "5":
                case "５":
                case "五":
                    result = 5;
                    break;

                case "6":
                case "６":
                case "六":
                    result = 6;
                    break;

                case "7":
                case "７":
                case "七":
                    result = 7;
                    break;

                case "8":
                case "８":
                case "八":
                    result = 8;
                    break;

                case "9":
                case "９":
                case "九":
                    result = 9;
                    break;

                default:
                    result = 0;
                    break;

            }

            return result;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 駒の表面の文字
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static string SyuruiToLabel(KomaSyurui syurui)
        {
            string koma;

            switch (syurui)
            {
                case KomaSyurui.Fu:
                    koma = "歩";
                    break;

                case KomaSyurui.Kyo:
                    koma = "香";
                    break;

                case KomaSyurui.Kei:
                    koma = "桂";
                    break;

                case KomaSyurui.Gin:
                    koma = "銀";
                    break;

                case KomaSyurui.Kin:
                    koma = "金";
                    break;

                case KomaSyurui.Hisya:
                    koma = "飛";
                    break;

                case KomaSyurui.Kaku:
                    koma = "角";
                    break;

                case KomaSyurui.Oh:
                    koma = "王";
                    break;

                case KomaSyurui.Tokin:
                    koma = "と";//と金
                    break;

                case KomaSyurui.NariKyo:
                    koma = "杏";//成香
                    break;

                case KomaSyurui.NariKei:
                    koma = "圭";//成桂
                    break;

                case KomaSyurui.NariGin:
                    koma = "全";//成銀
                    break;

                case KomaSyurui.Ryu:
                    koma = "竜";
                    break;

                case KomaSyurui.Uma:
                    koma = "馬";
                    break;

                case KomaSyurui.CTRL_NOTHING_PROPERTY:
                    koma = "Ｕ種";
                    break;

                default:
                    koma = "×";
                    break;
            }

            return koma;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 成り駒なら真。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static bool IsSyuruiNari(KomaSyurui syurui)
        {
            bool isNari;

            switch (syurui)
            {
                case KomaSyurui.Tokin:
                case KomaSyurui.NariKyo:
                case KomaSyurui.NariKei:
                case KomaSyurui.NariGin:
                case KomaSyurui.Ryu:
                case KomaSyurui.Uma:
                    isNari = true;
                    break;
                default:
                    isNari = false;
                    break;
            }

            return isNari;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 棋譜の符号用の駒の文字
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static string SyuruiToFugo(KomaSyurui syurui)
        {
            string koma;

            switch (syurui)
            {
                case KomaSyurui.Fu:
                    koma = "歩";
                    break;

                case KomaSyurui.Kyo:
                    koma = "香";
                    break;

                case KomaSyurui.Kei:
                    koma = "桂";
                    break;

                case KomaSyurui.Gin:
                    koma = "銀";
                    break;

                case KomaSyurui.Kin:
                    koma = "金";
                    break;

                case KomaSyurui.Hisya:
                    koma = "飛";
                    break;

                case KomaSyurui.Kaku:
                    koma = "角";
                    break;

                case KomaSyurui.Oh:
                    koma = "王";
                    break;

                case KomaSyurui.Tokin:
                    koma = "と";
                    break;

                case KomaSyurui.NariKyo:
                    koma = "成香";
                    break;

                case KomaSyurui.NariKei:
                    koma = "成桂";
                    break;

                case KomaSyurui.NariGin:
                    koma = "成銀";
                    break;

                case KomaSyurui.Ryu:
                    koma = "竜";
                    break;

                case KomaSyurui.Uma:
                    koma = "馬";
                    break;

                case KomaSyurui.CTRL_NOTHING_PROPERTY:
                    koma = "Ｕ符";
                    break;

                default:
                    koma = "×";
                    break;
            }

            return koma;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 棋譜の符号用の駒の文字。先手玉→K、後手玉→k など。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static string SyuruiToSfen(Sengo sengo, KomaSyurui syurui)
        {
            string koma;

            switch (syurui)
            {
                case KomaSyurui.Fu:
                    koma = "P";
                    break;

                case KomaSyurui.Kyo:
                    koma = "L";
                    break;

                case KomaSyurui.Kei:
                    koma = "N";
                    break;

                case KomaSyurui.Gin:
                    koma = "S";
                    break;

                case KomaSyurui.Kin:
                    koma = "G";
                    break;

                case KomaSyurui.Hisya:
                    koma = "R";
                    break;

                case KomaSyurui.Kaku:
                    koma = "B";
                    break;

                case KomaSyurui.Oh:
                    koma = "K";
                    break;

                case KomaSyurui.Tokin:
                    koma = "+P";
                    break;

                case KomaSyurui.NariKyo:
                    koma = "+L";
                    break;

                case KomaSyurui.NariKei:
                    koma = "+N";
                    break;

                case KomaSyurui.NariGin:
                    koma = "+S";
                    break;

                case KomaSyurui.Ryu:
                    koma = "+R";
                    break;

                case KomaSyurui.Uma:
                    koma = "+B";
                    break;

                case KomaSyurui.CTRL_NOTHING_PROPERTY:
                    koma = "ＵSfen";
                    break;

                default:
                    koma = "×";
                    break;
            }

            if (Sengo.Gote == sengo)
            {
                koma = koma.ToLower();
            }

            return koma;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 棋譜の符号用の駒の文字。打用。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static string SyuruiToSfenDa(KomaSyurui syurui)
        {
            string koma;

            switch (syurui)
            {
                case KomaSyurui.Fu:
                case KomaSyurui.Tokin:
                    koma = "P";
                    break;

                case KomaSyurui.Kyo:
                case KomaSyurui.NariKyo:
                    koma = "L";
                    break;

                case KomaSyurui.Kei:
                case KomaSyurui.NariKei:
                    koma = "N";
                    break;

                case KomaSyurui.Gin:
                case KomaSyurui.NariGin:
                    koma = "S";
                    break;

                case KomaSyurui.Kin:
                    koma = "G";
                    break;

                case KomaSyurui.Hisya:
                case KomaSyurui.Ryu:
                    koma = "R";
                    break;

                case KomaSyurui.Kaku:
                case KomaSyurui.Uma:
                    koma = "B";
                    break;

                case KomaSyurui.Oh:
                    koma = "K";
                    break;

                case KomaSyurui.CTRL_NOTHING_PROPERTY:
                    koma = "＜打Ｕ＞";
                    break;

                default:
                    koma = "＜打×＞";
                    break;
            }

            return koma;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 駒の種類。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static void SfenSyokihaichiToSyurui(string sfen, out Sengo sengo, out KomaSyurui syurui)
        {
            switch (sfen)
            {
                case "P":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Fu;
                    break;

                case "p":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Fu;
                    break;

                case "L":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Kyo;
                    break;

                case "l":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Kyo;
                    break;

                case "N":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Kei;
                    break;

                case "n":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Kei;
                    break;

                case "S":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Gin;
                    break;

                case "s":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Gin;
                    break;

                case "G":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Kin;
                    break;

                case "g":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Kin;
                    break;

                case "R":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Hisya;
                    break;

                case "r":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Hisya;
                    break;

                case "B":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Kaku;
                    break;

                case "b":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Kaku;
                    break;

                case "K":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Oh;
                    break;

                case "k":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Oh;
                    break;

                case "+P":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Tokin;
                    break;

                case "+p":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Tokin;
                    break;

                case "+L":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.NariKyo;
                    break;

                case "+l":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.NariKyo;
                    break;

                case "+N":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.NariKei;
                    break;

                case "+n":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.NariKei;
                    break;

                case "+S":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.NariGin;
                    break;

                case "+s":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.NariGin;
                    break;

                case "+R":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Hisya;
                    break;

                case "+r":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Hisya;
                    break;

                case "+B":
                    sengo = Sengo.Sente;
                    syurui = KomaSyurui.Kaku;
                    break;

                case "+b":
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.Kaku;
                    break;

                default:
                    sengo = Sengo.Gote;
                    syurui = KomaSyurui.CTRL_NOTHING_PROPERTY;
                    break;
            }
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 打った駒の種類。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static void SfenUttaSyurui(string sfen, out KomaSyurui syurui)
        {
            switch (sfen)
            {
                case "P":
                    syurui = KomaSyurui.Fu;
                    break;

                case "L":
                    syurui = KomaSyurui.Kyo;
                    break;

                case "N":
                    syurui = KomaSyurui.Kei;
                    break;

                case "S":
                    syurui = KomaSyurui.Gin;
                    break;

                case "G":
                    syurui = KomaSyurui.Kin;
                    break;

                case "R":
                    syurui = KomaSyurui.Hisya;
                    break;

                case "B":
                    syurui = KomaSyurui.Kaku;
                    break;

                case "K":
                    syurui = KomaSyurui.Oh;
                    break;

                case "+P":
                    syurui = KomaSyurui.Tokin;
                    break;

                case "+L":
                    syurui = KomaSyurui.NariKyo;
                    break;

                case "+N":
                    syurui = KomaSyurui.NariKei;
                    break;

                case "+S":
                    syurui = KomaSyurui.NariGin;
                    break;

                case "+R":
                    syurui = KomaSyurui.Hisya;
                    break;

                case "+B":
                    syurui = KomaSyurui.Kaku;
                    break;

                default:
                    syurui = KomaSyurui.CTRL_NOTHING_PROPERTY;
                    break;
            }
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 駒の文字を、列挙型へ変換。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="moji"></param>
        /// <returns></returns>
        public static KomaSyurui MojiToSyurui(string moji)
        {
            KomaSyurui syurui;

            switch (moji)
            {
                case "歩":
                    syurui = KomaSyurui.Fu;
                    break;

                case "香":
                    syurui = KomaSyurui.Kyo;
                    break;

                case "桂":
                    syurui = KomaSyurui.Kei;
                    break;

                case "銀":
                    syurui = KomaSyurui.Gin;
                    break;

                case "金":
                    syurui = KomaSyurui.Kin;
                    break;

                case "飛":
                    syurui = KomaSyurui.Hisya;
                    break;

                case "角":
                    syurui = KomaSyurui.Kaku;
                    break;

                case "王"://thru
                case "玉":
                    syurui = KomaSyurui.Oh;
                    break;

                case "と":
                    syurui = KomaSyurui.Tokin;
                    break;

                case "成香":
                    syurui = KomaSyurui.NariKyo;
                    break;

                case "成桂":
                    syurui = KomaSyurui.NariKei;
                    break;

                case "成銀":
                    syurui = KomaSyurui.NariGin;
                    break;

                case "竜"://thru
                case "龍":
                    syurui = KomaSyurui.Ryu;
                    break;

                case "馬":
                    syurui = KomaSyurui.Uma;
                    break;

                default:
                    syurui = KomaSyurui.CTRL_NOTHING_PROPERTY;
                    break;
            }

            return syurui;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 種類を見て、成／不成を判定
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static bool SyuruiToNari(KomaSyurui syurui)
        {
            bool nari;

            switch (syurui)
            {
                //----------
                // 成
                //----------
                case KomaSyurui.Tokin: //と金
                case KomaSyurui.NariKyo: //成香
                case KomaSyurui.NariKei: //成桂
                case KomaSyurui.NariGin: //成銀
                case KomaSyurui.Ryu: //竜
                case KomaSyurui.Uma: //馬
                    nari = true;
                    break;

                //----------
                // 不成
                //----------
                case KomaSyurui.Fu: //歩
                case KomaSyurui.Kyo: //香
                case KomaSyurui.Kei: //桂
                case KomaSyurui.Gin: //銀
                case KomaSyurui.Kin: //金
                case KomaSyurui.Hisya: //飛
                case KomaSyurui.Kaku: //角
                case KomaSyurui.Oh: //王
                default:
                    nari = false;
                    break;
            }

            return nari;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 種類を見て、不成の種類を判定
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static KomaSyurui SyuruiToFunariCase(KomaSyurui syurui)
        {
            KomaSyurui funari;

            switch (syurui)
            {
                //----------
                // 成 → 不成
                //----------
                case KomaSyurui.Tokin: //と金
                    funari = KomaSyurui.Fu;
                    break;

                case KomaSyurui.NariKyo: //成香
                    funari = KomaSyurui.Kyo;
                    break;

                case KomaSyurui.NariKei: //成桂
                    funari = KomaSyurui.Kei;
                    break;

                case KomaSyurui.NariGin: //成銀
                    funari = KomaSyurui.Gin;
                    break;

                case KomaSyurui.Ryu: //竜
                    funari = KomaSyurui.Hisya;
                    break;

                case KomaSyurui.Uma: //馬
                    funari = KomaSyurui.Kaku;
                    break;

                //----------
                // 不成
                //----------
                case KomaSyurui.Fu: //歩
                case KomaSyurui.Kyo: //香
                case KomaSyurui.Kei: //桂
                case KomaSyurui.Gin: //銀
                case KomaSyurui.Kin: //金
                case KomaSyurui.Hisya: //飛
                case KomaSyurui.Kaku: //角
                case KomaSyurui.Oh: //王
                default:
                    funari = syurui;
                    break;
            }

            return funari;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 種類を見て、成りの種類を判定
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="syurui"></param>
        /// <returns></returns>
        public static KomaSyurui SyuruiToNariCase(KomaSyurui syurui)
        {
            KomaSyurui nari;

            switch (syurui)
            {
                //----------
                // 
                //----------

                //----------
                // 不成→成
                //----------
                case KomaSyurui.Fu: //歩
                    nari = KomaSyurui.Tokin;
                    break;

                case KomaSyurui.Kyo: //香
                    nari = KomaSyurui.NariKyo;
                    break;

                case KomaSyurui.Kei: //桂
                    nari = KomaSyurui.NariKei;
                    break;

                case KomaSyurui.Gin: //銀
                    nari = KomaSyurui.NariGin;
                    break;

                case KomaSyurui.Hisya: //飛
                    nari = KomaSyurui.Ryu;
                    break;

                case KomaSyurui.Kaku: //角
                    nari = KomaSyurui.Uma;
                    break;

                //----------
                // 成　or　成のない駒
                //----------
                case KomaSyurui.Tokin: //と金
                case KomaSyurui.NariKyo: //成香
                case KomaSyurui.NariKei: //成桂
                case KomaSyurui.NariGin: //成銀
                case KomaSyurui.Ryu: //竜
                case KomaSyurui.Uma: //馬

                case KomaSyurui.Kin: //金
                case KomaSyurui.Oh: //王
                default:
                    nari = syurui;
                    break;
            }

            return nari;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 打。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="da"></param>
        /// <returns></returns>
        public static string BoolToDa(DaHyoji da)
        {
            string daStr = "";

            if (DaHyoji.Visible == da)
            {
                daStr = "打";
            }

            return daStr;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 打表示。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="daStr"></param>
        /// <returns></returns>
        public static DaHyoji StrToDaHyoji(string daStr)
        {
            DaHyoji daHyoji = DaHyoji.No_Print;

            if (daStr == "打")
            {
                daHyoji = DaHyoji.Visible;
            }

            return daHyoji;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 成り
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="nari"></param>
        /// <returns></returns>
        public static string NariToStr(NariFunari nari)
        {
            string nariStr = "";

            switch (nari)
            {
                case NariFunari.Nari:
                    nariStr = "成";
                    break;
                case NariFunari.Funari:
                    nariStr = "不成";
                    break;
                default:
                    break;
            }

            return nariStr;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 成り。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="nariStr"></param>
        /// <returns></returns>
        public static NariFunari NariToBool(string nariStr)
        {
            NariFunari nari;

            if ("成" == nariStr)
            {
                nari = NariFunari.Nari;
            }
            else if ("不成" == nariStr)
            {
                nari = NariFunari.Funari;
            }
            else
            {
                nari = NariFunari.CTRL_SONOMAMA;
            }

            return nari;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 先後。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengoStr"></param>
        /// <returns></returns>
        public static Sengo SengoToEnum(string sengoStr)
        {
            Sengo sengo;

            switch (sengoStr)
            {
                case "△":
                    sengo = Sengo.Gote;
                    break;

                case "▲":
                default:
                    sengo = Sengo.Sente;
                    break;
            }

            return sengo;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 先後。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <returns></returns>
        public static string SengoToStr(Sengo sengo)
        {
            string sengoStr;

            switch (sengo)
            {
                case Sengo.Gote:
                    sengoStr = "△";
                    break;

                case Sengo.Sente:
                default:
                    sengoStr = "▲";
                    break;
            }

            return sengoStr;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 先後。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <returns></returns>
        public static string SengoToKanji(Sengo sengo)
        {
            string sengoStr;

            switch (sengo)
            {
                case Sengo.Sente:
                    sengoStr = "先手";
                    break;
                case Sengo.Gote:
                    sengoStr = "後手";
                    break;
                default:
                    sengoStr = "×";
                    break;
            }

            return sengoStr;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 右左。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="migiHidari"></param>
        /// <returns></returns>
        public static string MigiHidariToStr(MigiHidari migiHidari)
        {
            string str;

            switch (migiHidari)
            {
                case MigiHidari.Migi:
                    str = "右";
                    break;

                case MigiHidari.Hidari:
                    str = "左";
                    break;

                case MigiHidari.Sugu:
                    str = "直";
                    break;

                default:
                    str = "";
                    break;
            }

            return str;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 寄、右、左、直、なし
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="migiHidariStr"></param>
        /// <returns></returns>
        public static MigiHidari StrToMigiHidari(string migiHidariStr)
        {
            MigiHidari migiHidari;

            switch (migiHidariStr)
            {
                case "右":
                    migiHidari = MigiHidari.Migi;
                    break;

                case "左":
                    migiHidari = MigiHidari.Hidari;
                    break;

                case "直":
                    migiHidari = MigiHidari.Sugu;
                    break;

                default:
                    migiHidari = MigiHidari.No_Print;
                    break;
            }

            return migiHidari;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 上がる、引く
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="agaruHiku"></param>
        /// <returns></returns>
        public static string AgaruHikuToStr(AgaruHiku agaruHiku)
        {
            string str;

            switch (agaruHiku)
            {
                case AgaruHiku.Yoru:
                    str = "寄";
                    break;

                case AgaruHiku.Hiku:
                    str = "引";
                    break;

                case AgaruHiku.Agaru:
                    str = "上";
                    break;

                default:
                    str = "";
                    break;
            }

            return str;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 上がる、引く。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="agaruHikuStr"></param>
        /// <returns></returns>
        public static AgaruHiku StrToAgaruHiku(string agaruHikuStr)
        {
            AgaruHiku agaruHiku;

            switch (agaruHikuStr)
            {
                case "寄":
                    agaruHiku = AgaruHiku.Yoru;
                    break;

                case "引":
                    agaruHiku = AgaruHiku.Hiku;
                    break;

                case "上":
                    agaruHiku = AgaruHiku.Agaru;
                    break;

                default:
                    agaruHiku = AgaruHiku.No_Print;
                    break;
            }

            return agaruHiku;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 先後の交代
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo">先後</param>
        /// <returns>ひっくりかえった先後</returns>
        public static Sengo AlternateSengo(Sengo sengo)
        {
            Sengo result;

            switch (sengo)
            {
                case Sengo.Sente:
                    result = Sengo.Gote;
                    break;

                case Sengo.Gote:
                    result = Sengo.Sente;
                    break;

                default:
                    result = sengo;
                    break;
            }

            return result;
        }

    }
}
