﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Xenon.KifuNarabe.L01_Communication;

namespace Xenon.KifuNarabe.L02_Common
{

    /// <summary>
    /// ************************************************************************************************************************
    /// 符号テキスト(*1)の、データの持ち方をここに書いています。
    /// ************************************************************************************************************************
    /// 
    ///         *1…「▲５五角右引」など。筋、段、同、駒種類、成、右左、上引、成、打。
    /// 
    /// </summary>
    public class Fugo
    {

        #region プロパティー類

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 駒種類
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public KomaSyurui KomaSyurui
        {
            get
            {
                return this.komaSyurui;
            }
        }
        private KomaSyurui komaSyurui;


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 右、左、直など
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public MigiHidari MigiHidari
        {
            get
            {
                return this.migiHidari;
            }
        }
        private MigiHidari migiHidari;


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 上、引
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public AgaruHiku AgaruHiku
        {
            get
            {
                return this.agaruHiku;
            }
        }
        private AgaruHiku agaruHiku;


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 成
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public NariFunari Nari
        {
            get
            {
                return this.nari;
            }
        }
        private NariFunari nari;


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// “打”表示。必ずしも持ち駒を打つタイミングとは一致しない。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public DaHyoji DaHyoji
        {
            get
            {
                return this.daHyoji;
            }
        }
        private DaHyoji daHyoji;

        #endregion



        /// <summary>
        /// ************************************************************************************************************************
        /// コンストラクターです。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="komaSyurui"></param>
        /// <param name="migiHidari"></param>
        /// <param name="agaruHiku"></param>
        /// <param name="nari"></param>
        /// <param name="daHyoji"></param>
        public Fugo(
            KomaSyurui komaSyurui, MigiHidari migiHidari, AgaruHiku agaruHiku, NariFunari nari, DaHyoji daHyoji)
        {
            this.komaSyurui = komaSyurui;
            this.migiHidari = migiHidari;
            this.agaruHiku = agaruHiku;
            this.nari = nari;
            this.daHyoji = daHyoji;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 棋譜用の符号テキスト(*1)を作ります。
        /// ************************************************************************************************************************
        /// 
        ///         *1…「▲５五銀上」など。
        /// 
        /// </summary>
        /// <param name="douExpr">“同”表記に置き換えるなら真。</param>
        /// <returns></returns>
        public string ToText(bool douExpr, TeProcess process, TeProcess previousMasu)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append(Converter01.SengoToStr(process.Sengo));

            //------------------------------
            // “同”か、“筋・段”か
            //------------------------------
            if (
                douExpr
                && null!=previousMasu
                && previousMasu.MatchMasu(process)
                )
            {
                sb.Append("同");
            }
            else
            {
                sb.Append(Converter01.IntToArabiaSuji(process.Suji));
                sb.Append(Converter01.IntToKanSuji(process.Dan));
            }

            //------------------------------
            // “歩”とか。“全”ではなく“成銀”    ＜符号用＞
            //------------------------------
            sb.Append(Converter01.SyuruiToFugo(this.KomaSyurui));

            //------------------------------
            // “右”とか
            //------------------------------
            sb.Append(Converter01.MigiHidariToStr(this.MigiHidari));

            //------------------------------
            // “寄”とか
            //------------------------------
            sb.Append(Converter01.AgaruHikuToStr(this.AgaruHiku));

            //------------------------------
            // “成”とか
            //------------------------------
            sb.Append(Converter01.NariToStr(this.Nari));

            //------------------------------
            // “打”とか
            //------------------------------
            sb.Append(Converter01.BoolToDa(this.DaHyoji));

            return sb.ToString();
        }

    }
}
