﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Text.RegularExpressions;
using Xenon.KifuNarabe.L01_Communication;
using Xenon.KifuNarabe.L05_Server;

namespace Xenon.KifuNarabe.L02_Common
{

    /// <summary>
    /// ************************************************************************************************************************
    /// 棋譜の、データの持ち方をここに書きます。
    /// ************************************************************************************************************************
    /// </summary>
    public class Kifu
    {

        #region プロパティー類

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// ランダム
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Random Random
        {
            get
            {
                return this.random;
            }
        }
        private Random random;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 一手前の筋
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int IttemaeSuji
        {
            get
            {
                return this.ittemaeSuji;
            }
        }
        private int ittemaeSuji;


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 一手前の段
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int IttemaeDan
        {
            get
            {
                return this.ittemaeDan;
            }
        }
        private int ittemaeDan;


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 初期配置。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public string Startpos
        {
            get
            {
                return this.startpos;
            }
        }
        private string startpos;


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 符号リスト
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public List<TeProcess> ProcessList
        {
            get
            {
                return this.processList;
            }
        }
        private List<TeProcess> processList;


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 局面。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Kyokumen Kyokumen
        {
            get
            {
                return this.kyokumen;
            }
        }
        private Kyokumen kyokumen;


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 最後の指し手。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public TeProcess Last
        {
            get
            {
                TeProcess last = null;

                if (0 != this.processList.Count)
                {
                    last = this.processList.Last();
                }

                return last;
            }
        }


        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 現在の先後
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Sengo Sengo
        {
            get
            {
                return this.sengo;
            }
        }
        private Sengo sengo;

        public void SetSengo(Sengo sengo)
        {
            this.sengo = sengo;
        }


        #endregion



        /// <summary>
        /// ************************************************************************************************************************
        /// 最初。
        /// ************************************************************************************************************************
        /// </summary>
        public void Init()
        {
            this.ittemaeSuji = TeProcess.CTRL_NOTHING_PROPERTY_SUJI;
            this.ittemaeDan = TeProcess.CTRL_NOTHING_PROPERTY_DAN;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 手番交替。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="lastSuji"></param>
        /// <param name="lastDan"></param>
        public void ChangeTurn(int lastSuji, int lastDan)
        {
            this.ittemaeSuji = lastSuji;
            this.ittemaeDan = lastDan;
            //System.Console.WriteLine("一手前筋：" + this.ittemaeSuji + "　一手前段：" + this.ittemaeDan);

            this.SetSengo(Converter01.AlternateSengo(this.Sengo));

            ShogiEngineService.ChangeTurn(this);
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// コンストラクターです。
        /// ************************************************************************************************************************
        /// </summary>
        public Kifu()
        {
            this.random = new Random(0);

            this.Init();

            this.sengo = Sengo.Sente;

            this.kyokumen = new Kyokumen();

            this.startpos = "未設定";
            this.processList = new List<TeProcess>();
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 平手の初期配置にします。
        /// ************************************************************************************************************************
        /// </summary>
        public void SetStartpos()
        {
            this.startpos = "startpos";
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 初期配置を指定します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="startpos"></param>
        public void SetStartpos(string startpos)
        {
            this.startpos = startpos;
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// [ここから採譜]機能
        /// ************************************************************************************************************************
        /// </summary>
        public void SetStartpos_KokokaraSaifu(Sengo sengo)
        {
            StringBuilder sb = new StringBuilder();

            for (int dan = 1; dan <= 9; dan++)
            {
                int spaceCount = 0;

                for (int suji = 9; suji >= 1; suji--)
                {
                    int hKoma = this.Kyokumen.HKoma_AtMasuDokodemo(Okiba.ShogiBan, new TeProcess(
                        Sengo.CTRL_NOTHING_PROPERTY,
                        Okiba.CTRL_NOTHING_PROPERTY,
                        suji,
                        dan,
                        KomaSyurui.CTRL_NOTHING_PROPERTY
                        ));

                    if (-1 != hKoma)
                    {
                        if (0 < spaceCount)
                        {
                            sb.Append(spaceCount);
                            spaceCount = 0;
                        }
                        sb.Append(Converter01.SyuruiToSfen(this.Kyokumen.KomaDoors[hKoma].Sengo, this.Kyokumen.KomaDoors[hKoma].Syurui));
                    }
                    else
                    {
                        spaceCount++;
                    }

                }

                if (0 < spaceCount)
                {
                    sb.Append(spaceCount);
                    spaceCount = 0;
                }

                if (dan != 9)
                {
                    sb.Append("/");
                }
            }

            sb.Append(" ");

            //------------------------------------------------------------
            // 先後
            //------------------------------------------------------------
            switch (sengo)
            {
                case Sengo.Gote:
                    sb.Append("w");
                    break;
                default:
                    sb.Append("b");
                    break;
            }

            sb.Append(" ");

            //------------------------------------------------------------
            // 持ち駒
            //------------------------------------------------------------
            {
                int mK = 0;
                int mR = 0;
                int mB = 0;
                int mG = 0;
                int mS = 0;
                int mN = 0;
                int mL = 0;
                int mP = 0;

                int mk = 0;
                int mr = 0;
                int mb = 0;
                int mg = 0;
                int ms = 0;
                int mn = 0;
                int ml = 0;
                int mp = 0;

                // 先手
                List<int> hKomasS = this.Kyokumen.HKomas_ByOkiba(Okiba.Sente_Komadai);
                foreach (int hKoma in hKomasS)
                {
                    switch (Converter01.SyuruiToFunariCase(this.Kyokumen.KomaDoors[hKoma].Syurui))
                    {
                        case KomaSyurui.Oh:
                            mK++;
                            break;
                        case KomaSyurui.Hisya:
                            mR++;
                            break;
                        case KomaSyurui.Kaku:
                            mB++;
                            break;
                        case KomaSyurui.Kin:
                            mG++;
                            break;
                        case KomaSyurui.Gin:
                            mS++;
                            break;
                        case KomaSyurui.Kei:
                            mN++;
                            break;
                        case KomaSyurui.Kyo:
                            mL++;
                            break;
                        case KomaSyurui.Fu:
                            mP++;
                            break;
                    }
                }

                // 後手
                List<int> hKomasG = this.Kyokumen.HKomas_ByOkiba(Okiba.Gote_Komadai);
                foreach (int hKoma in hKomasG)
                {
                    switch (Converter01.SyuruiToFunariCase(this.Kyokumen.KomaDoors[hKoma].Syurui))
                    {
                        case KomaSyurui.Oh:
                            mk++;
                            break;
                        case KomaSyurui.Hisya:
                            mr++;
                            break;
                        case KomaSyurui.Kaku:
                            mb++;
                            break;
                        case KomaSyurui.Kin:
                            mg++;
                            break;
                        case KomaSyurui.Gin:
                            ms++;
                            break;
                        case KomaSyurui.Kei:
                            mn++;
                            break;
                        case KomaSyurui.Kyo:
                            ml++;
                            break;
                        case KomaSyurui.Fu:
                            mp++;
                            break;
                    }
                }

                if (0 == mK + mR + mB + mG + mS + mN + mL + mP + mk + mr + mb + mg + ms + mn + ml + mp)
                {
                    sb.Append("-");
                }
                else
                {
                    if (0 < mK)
                    {
                        sb.Append("K");
                        sb.Append(mK);
                    }

                    if (0 < mR)
                    {
                        sb.Append("R");
                        sb.Append(mR);
                    }

                    if (0 < mB)
                    {
                        sb.Append("B");
                        sb.Append(mB);
                    }

                    if (0 < mG)
                    {
                        sb.Append("G");
                        sb.Append(mG);
                    }

                    if (0 < mS)
                    {
                        sb.Append("S");
                        sb.Append(mS);
                    }

                    if (0 < mN)
                    {
                        sb.Append("N");
                        sb.Append(mN);
                    }

                    if (0 < mL)
                    {
                        sb.Append("L");
                        sb.Append(mL);
                    }

                    if (0 < mP)
                    {
                        sb.Append("P");
                        sb.Append(mP);
                    }

                    if (0 < mk)
                    {
                        sb.Append("k");
                        sb.Append(mk);
                    }

                    if (0 < mr)
                    {
                        sb.Append("r");
                        sb.Append(mr);
                    }

                    if (0 < mb)
                    {
                        sb.Append("b");
                        sb.Append(mb);
                    }

                    if (0 < mg)
                    {
                        sb.Append("g");
                        sb.Append(mg);
                    }

                    if (0 < ms)
                    {
                        sb.Append("s");
                        sb.Append(ms);
                    }

                    if (0 < mn)
                    {
                        sb.Append("n");
                        sb.Append(mn);
                    }

                    if (0 < ml)
                    {
                        sb.Append("l");
                        sb.Append(ml);
                    }

                    if (0 < mp)
                    {
                        sb.Append("p");
                        sb.Append(mp);
                    }
                }

            }

            // 手目
            sb.Append(" 1");

            //------------------------------------------------------------
            // 棋譜を空に
            //------------------------------------------------------------
            this.ProcessList.Clear();

            this.SetStartpos(sb.ToString());
        }



        /// <summary>
        /// ************************************************************************************************************************
        /// 棋譜に符号を追加します。
        /// ************************************************************************************************************************
        /// 
        /// ①コマ送り用。
        /// ②「成り」フラグの更新用。
        /// ③マウス操作用
        /// 
        /// </summary>
        /// <param name="process"></param>
        public void Add(TeProcess process)
        {
            this.processList.Add(process);
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 最後の要素を切り取って返します。なければヌル。
        /// ************************************************************************************************************************
        /// </summary>
        /// <returns></returns>
        public void PopLast(out TeProcess last)
        {
            last = this.Last;

            if (null != last)
            {
                this.processList.RemoveAt(this.processList.Count - 1);
            }
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 棋譜を空っぽにします。
        /// ************************************************************************************************************************
        /// </summary>
        public void Clear()
        {
            this.processList.Clear();
            this.SetSengo(Sengo.Sente);
        }
    }


}
