﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Xenon.KifuNarabe.L01_Communication;


namespace Xenon.KifuNarabe.L02_Common
{

    /// <summary>
    /// ************************************************************************************************************************
    /// 駒の軌道を、ここに書きます。
    /// ************************************************************************************************************************
    /// </summary>
    public abstract class KomanoKidou
    {


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、飛車の(自)升の候補（Ａ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcA_Hisya(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcA = new List<TeProcess>();

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │至│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A7│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //　                  Ａ

                for (int i = 0; i < 8; i++) { srcA.Add( itaru.Offset(sengo, 0, i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //　                  Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A7│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │至│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcA.Add( itaru.Offset(sengo, 0, -(i + 1))); }
            }

            return srcA;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、飛車の(自)升の候補（Ｃ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcC_Hisya(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcC = new List<TeProcess>();

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C7│C6│C5│C4│C3│C2│C1│C0│至│
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcC.Add( itaru.Offset(sengo, i + 1, 0)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│G0│G1│G2│G3│G4│G5│G6│G7│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcC.Add( itaru.Offset(sengo, -(i + 1), 0)); }
            }

            return srcC;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、飛車の(自)升の候補（Ｅ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcE_Hisya(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcE = new List<TeProcess>();

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //　                  Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E7│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │至│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcE.Add( itaru.Offset(sengo, 0, -(i + 1))); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │至│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E7│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //　                  Ｅ

                for (int i = 0; i < 8; i++) { srcE.Add( itaru.Offset(sengo, 0, i + 1)); }
            }

            return srcE;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、飛車の(自)升の候補（Ｇ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcG_Hisya(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcG = new List<TeProcess>();

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│G0│G1│G2│G3│G4│G5│G6│G7│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcG.Add( itaru.Offset(sengo, -(i + 1), 0)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G7│G6│G5│G4│G3│G2│G1│G0│至│
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcG.Add( itaru.Offset(sengo, i + 1, 0)); }
            }

            return srcG;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、角の(自)升の候補（Ｂ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcB_Kaku(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcB = new List<TeProcess>();

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│  │  │  │  │  │  │  │  │至│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │B0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │B2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │B3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B7│  │  │  │  │  │  │  │  │
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcB.Add( itaru.Offset(sengo, i + 1, i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│  │  │  │  │  │  │  │  │B7│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │B6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │B5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │B4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │B3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│  │  │  │  │  │  │  │  │
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcB.Add( itaru.Offset(sengo, -(i + 1), -(i + 1))); }
            }

            return srcB;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、角の(自)升の候補（Ｄ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcD_Kaku(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcD = new List<TeProcess>();

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│D7│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │D3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │D2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │D0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │至│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcD.Add( itaru.Offset(sengo, i + 1, -(i + 1))); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //　┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│至│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │D3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │D4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │D5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │D6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │D7│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                for (int i = 0; i < 8; i++) { srcD.Add( itaru.Offset(sengo, -(i + 1), i + 1)); }
            }

            return srcD;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、角の(自)升の候補（Ｆ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcF_Kaku(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcF = new List<TeProcess>();

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│  │  │  │  │  │  │  │  │F7│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │F6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │F5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │F4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │F3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│  │  │  │  │  │  │  │  │
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcF.Add( itaru.Offset(sengo, -(i + 1), -(i + 1))); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│  │  │  │  │  │  │  │  │至│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │F0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │F2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │F3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F7│  │  │  │  │  │  │  │  │
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcF.Add( itaru.Offset(sengo, i + 1, i + 1)); }
            }

            return srcF;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、角の(自)升の候補（Ｈ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcH_Kaku(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcH = new List<TeProcess>();

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //　┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│至│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │H3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │H4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │H5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │H6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │H7│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                for (int i = 0; i < 8; i++) { srcH.Add( itaru.Offset(sengo, -(i + 1), i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│H7│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │H3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │H2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │H0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │至│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcH.Add( itaru.Offset(sengo, i + 1, -(i + 1))); }
            }

            return srcH;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ａ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcA_Oh(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcA;

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│  │Ａ│  │
                //└─┴─┴─┘

                srcA = new List<TeProcess>() { itaru.Offset(sengo, 0, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│  │Ａ│  │
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│  │  │  │
                //└─┴─┴─┘

                srcA = new List<TeProcess>() { itaru.Offset(sengo, 0, -1) };
            }

            return srcA;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ｂ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcB_Oh(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcB;

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│Ｂ│  │  │
                //└─┴─┴─┘

                srcB = new List<TeProcess>() { itaru.Offset(sengo, 1, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│  │  │Ｂ│
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcB = new List<TeProcess>() { itaru.Offset(sengo, -1, -1) };
            }

            return srcB;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ｃ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcC_Oh(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcC;

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│Ｃ│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcC = new List<TeProcess>() { itaru.Offset(sengo, 1, 0) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│　│至│Ｃ│
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcC = new List<TeProcess>() { itaru.Offset(sengo, -1, 0) };
            }

            return srcC;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ｄ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcD_Oh(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcD;

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│Ｄ│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcD = new List<TeProcess>() { itaru.Offset(sengo, 1, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │Ｄ│
                //└─┴─┴─┘

                srcD = new List<TeProcess>() { itaru.Offset(sengo, -1, 1) };
            }

            return srcD;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ｅ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcE_Oh(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcE;


            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│　│Ｅ│  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcE = new List<TeProcess>() { itaru.Offset(sengo, 0, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│Ｅ│  │
                //└─┴─┴─┘

                srcE = new List<TeProcess>() { itaru.Offset(sengo, 0, 1) };
            }


            return srcE;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ｆ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcF_Oh(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcF;

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │Ｆ│
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcF = new List<TeProcess>() { itaru.Offset(sengo, -1, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │Ｆ│  │  │
                //  └─┴─┴─┘

                srcF = new List<TeProcess>() { itaru.Offset(sengo, 1, 1) };
            }

            return srcF;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ｇ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcG_Oh(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcG;

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │  │至│Ｇ│
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcG = new List<TeProcess>() { itaru.Offset(sengo, -1, 0) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │Ｇ│至│  │
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcG = new List<TeProcess>() { itaru.Offset(sengo, 1, 0) };
            }

            return srcG;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ｈ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcH_Oh(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcH;

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │  │  │Ｈ│
                //  └─┴─┴─┘

                srcH = new List<TeProcess>() { itaru.Offset(sengo, -1, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┐
                //  │Ｈ│  │  │
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcH = new List<TeProcess>() { itaru.Offset(sengo, 1, -1) };
            }

            return srcH;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ｉ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcI_Kei(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcI;

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┐　┌─┐
                //│Ｉ│　│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┘　└─┘

                srcI = new List<TeProcess>() { itaru.Offset(sengo, 1, -2) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┐　┌─┐
                //│  │　│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│  │　│Ｉ│
                //└─┘　└─┘

                srcI = new List<TeProcess>() { itaru.Offset(sengo, -1, 2) };
            }

            return srcI;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// (至)升を指定することで、王の(自)升の候補（Ｊ方向）全てを返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="itaru"></param>
        /// <returns></returns>
        public static List<TeProcess> SrcJ_Kei(Sengo sengo, TeProcess itaru)
        {
            List<TeProcess> srcJ;

            if (Sengo.Gote == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┐　┌─┐
                //│  │　│Ｊ│
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│  │  │  │
                //└─┘　└─┘

                //----------
                // 競合駒マス(pre masu)
                //----------
                srcJ = new List<TeProcess>() { itaru.Offset(sengo, -1, -2) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┐　┌─┐
                //│  │　│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│Ｊ│　│  │
                //└─┘　└─┘

                srcJ = new List<TeProcess>() { itaru.Offset(sengo, 1, 2) };
            }

            return srcJ;
        }

    }

}
