﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Reflection;
using System.Text.RegularExpressions;

using Xenon.KifuNarabe.L01_Communication;
using Xenon.KifuNarabe.L02_Common;

namespace Xenon.KifuNarabe.L03_TuginoItte
{

    /// <summary>
    /// ************************************************************************************************************************
    /// 符号(*1)を元に、棋譜データの持ち方に変換します。
    /// ************************************************************************************************************************
    /// 
    ///         *1…「▲７六歩」といった記述。
    ///         
    /// </summary>
    public abstract class TuginoItte_JapanFugo
    {

        /// <summary>
        /// ************************************************************************************************************************
        /// テキスト形式の符号「▲７六歩△３四歩▲２六歩…」の最初の要素を、切り取ってプロセスに変換します。
        /// ************************************************************************************************************************
        /// 
        /// 再生、コマ送りで利用。
        /// </summary>
        /// <returns></returns>
        public static bool GetData_FromText(
            string text, out string restText, out TeProcess process, Kifu kifu)
        {
            process = null;

            bool successful = false;

            restText = text;

            //------------------------------------------------------------
            // リスト作成
            //------------------------------------------------------------
            Regex regex = new Regex(
                @"^\s*([▲△]?)(?:([123456789１２３４５６７８９])([123456789１２３４５６７８９一二三四五六七八九]))?(同)?[\s　]*(歩|香|桂|銀|金|飛|角|王|玉|と|成香|成桂|成銀|竜|龍|馬)(右|左|直)?(寄|引|上)?(成|不成)?(打?)",
                RegexOptions.Singleline
            );

            MatchCollection mc = regex.Matches(text);
            foreach (Match m in mc)
            {
                if (0 < m.Groups.Count)
                {
                    successful = true;

                    // 残りのテキスト
                    restText = text.Substring(0, m.Index) + text.Substring(m.Index + m.Length, text.Length - (m.Index + m.Length));

                    TuginoItte_JapanFugo.GetData_FromTextSub(
                        m.Groups[1].Value,  //▲△
                        m.Groups[2].Value,  //123…9、１２３…９、一二三…九
                        m.Groups[3].Value,  //123…9、１２３…９、一二三…九
                        m.Groups[4].Value,  // “同”
                        m.Groups[5].Value,  //(歩|香|桂|…
                        m.Groups[6].Value,           // 右|左…
                        m.Groups[7].Value,  // 上|引
                        m.Groups[8].Value, //成|不成
                        m.Groups[9].Value,  //打
                        out process,
                        kifu.Kyokumen,
                        kifu
                        );
                }

                // 最初の１件だけ処理して終わります。
                break;
            }

            restText = restText.Trim();

            return successful;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 次の１手データを作ります(*1)
        /// ************************************************************************************************************************
        /// 
        ///         *1…符号１「▲６８銀上」を元に、「7968」を作ります。
        /// 
        /// ＜再生、コマ送りで呼び出されます＞
        /// </summary>
        /// <returns></returns>
        private static void GetData_FromTextSub(
            string strSengo, //▲△
            string strSuji, //123…9、１２３…９、一二三…九
            string strDan, //123…9、１２３…９、一二三…九
            string strDou, // “同”
            string strSrcSyurui, //(歩|香|桂|…
            string strMigiHidari,           // 右|左…
            string strAgaruHiku, // 上|引
            string strNariFunari, //成|不成
            string strDaHyoji, //打
            out TeProcess process,
            Kyokumen kyokumen,
            Kifu kifu
            )
        {
            //------------------------------
            // 符号確定
            //------------------------------
            MigiHidari migiHidari = Converter01.StrToMigiHidari(strMigiHidari);
            AgaruHiku agaruHiku = Converter01.StrToAgaruHiku(strAgaruHiku);            // 上|引
            NariFunari nariFunari = Converter01.NariToBool(strNariFunari);//成
            DaHyoji daHyoji = Converter01.StrToDaHyoji(strDaHyoji);             //打

            KomaSyurui srcSyurui = Converter01.MojiToSyurui(strSrcSyurui);


            //------------------------------
            // 
            //------------------------------
            Sengo sengo = Converter01.SengoToEnum(strSengo);


            int suji;
            int dan;

            if ("同" == strDou)
            {
                // 1手前の筋、段を求めます。
                suji = kifu.IttemaeSuji;
                dan = kifu.IttemaeDan;
            }
            else
            {
                suji = Converter01.SujiToInt(strSuji);
                dan = Converter01.SujiToInt(strDan);
            }

            // TODO: 駒台にあるかもしれない。
            Okiba srcOkiba = Okiba.ShogiBan; //Okiba.FUMEI_FUGO_YOMI_CHOKUGO;// Okiba.SHOGIBAN;
            if (DaHyoji.Visible == daHyoji)
            {
                if (Sengo.Gote == sengo)
                {
                    srcOkiba = Okiba.Gote_Komadai;
                }
                else
                {
                    srcOkiba = Okiba.Sente_Komadai;
                }
            }

            TeProcess dst1 = new TeProcess(
                sengo,
                Okiba.ShogiBan,
                //srcOkiba,
                suji,
                dan,
                //Process.ITTEMAE_KYOKUMEN_SUJI, // 符号からは、元位置は分かりません。
                //Process.ITTEMAE_KYOKUMEN_DAN, // 符号からは、元位置は分かりません。
                srcSyurui
                //KomaSyurui.UNKNOWN,
                //KomaSyurui.UNKNOWN//取った駒は分からない
                );

            int hFoundKoma = -1;

            //----------
            // 駒台の駒を(明示的に)打つなら
            //----------
            bool utsu = false;//駒台の駒を打つなら真
            if (DaHyoji.Visible == daHyoji)
            {
                utsu = true;
                goto gt_EndShogiban;
            }


            switch (srcSyurui)
            {
                #region 歩
                //************************************************************
                // 歩
                //************************************************************
                case KomaSyurui.Fu:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┬─┬─┐
                        //│  │  │  │
                        //├─┼─┼─┤
                        //│  │至│  │
                        //├─┼─┼─┤
                        //│  │Ｅ│  │
                        //└─┴─┴─┘
                        bool isE = true;

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }

                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 飛
                //************************************************************
                // 飛
                //************************************************************
                case KomaSyurui.Hisya:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │  │  │  │  │  │  │  │  │A7│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │A3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A0│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │G7│G6│G5│G4│G3│G2│G1│G0│至│C0│C1│C2│C3│C4│C5│C6│C7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E0│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │E4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E7│  │  │  │  │  │  │  │  │
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        bool isA = true;
                        bool isC = true;
                        bool isE = true;
                        bool isG = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isA = false;
                                isE = false;
                                break;
                            case AgaruHiku.Agaru:
                                isA = false;
                                isC = false;
                                isG = false;
                                break;
                            case AgaruHiku.Hiku:
                                isC = false;
                                isE = false;
                                isG = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isA = false;
                                isE = false;
                                isG = false;
                                break;
                            case MigiHidari.Hidari:
                                isA = false;
                                isC = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isA = false;
                                isC = false;
                                isG = false;
                                break;
                        }

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Hisya(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Hisya(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Hisya(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Hisya(sengo, dst1)); }

                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 角
                //************************************************************
                // 角
                //************************************************************
                case KomaSyurui.Kaku:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │H7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │B7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │H6│  │  │  │  │  │  │  │  │  │  │  │  │  │B6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │H5│  │  │  │  │  │  │  │  │  │  │  │B5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │H4│  │  │  │  │  │  │  │  │  │B4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │H3│  │  │  │  │  │  │  │B3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │H2│  │  │  │  │  │B2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │H1│  │  │  │B1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │H0│  │B0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │至│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │F0│  │D0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │F1│  │  │  │D1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │F2│  │  │  │  │  │D2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │F3│  │  │  │  │  │  │  │D3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │F4│  │  │  │  │  │  │  │  │  │D4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │F5│  │  │  │  │  │  │  │  │  │  │  │D5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │F6│  │  │  │  │  │  │  │  │  │  │  │  │  │D6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │F7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │D7│
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        bool isB = true;
                        bool isD = true;
                        bool isF = true;
                        bool isH = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isB = false;
                                isD = false;
                                isF = false;
                                isH = false;
                                break;
                            case AgaruHiku.Agaru:
                                isB = false;
                                isH = false;
                                break;
                            case AgaruHiku.Hiku:
                                isB = false;
                                isH = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isF = false;
                                isH = false;
                                break;
                            case MigiHidari.Hidari:
                                isB = false;
                                isD = false;
                                break;
                            case MigiHidari.Sugu:
                                isD = false;
                                isF = false;
                                break;
                        }

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Kaku(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Kaku(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Kaku(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Kaku(sengo, dst1)); }

                        //----------
                        // 候補マスＢ
                        //----------
                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 香
                //************************************************************
                // 香
                //************************************************************
                case KomaSyurui.Kyo:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┐
                        //  │  │至│  │
                        //  ├─┼─┼─┤
                        //  │  │E0│  │
                        //  ├─┼─┼─┤
                        //  │  │E1│  │
                        //  ├─┼─┼─┤
                        //  │  │E2│  │
                        //  ├─┼─┼─┤
                        //  │  │E3│  │
                        //  ├─┼─┼─┤
                        //  │  │E4│  │
                        //  ├─┼─┼─┤
                        //  │  │E5│  │
                        //  ├─┼─┼─┤
                        //  │  │E6│  │
                        //  ├─┼─┼─┤
                        //  │  │E7│  │
                        //  └─┴─┴─┘
                        bool isE = true;

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Hisya(sengo, dst1)); }

                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 桂
                //************************************************************
                // 桂
                //************************************************************
                case KomaSyurui.Kei:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┐　┌─┐
                        //│  │　│  │
                        //├─┼─┼─┤
                        //│　│  │  │
                        //├─┼─┼─┤
                        //│　│至│  │先手から見た図
                        //├─┼─┼─┤
                        //│　│  │  │
                        //├─┼─┼─┤
                        //│Ｊ│　│Ｉ│
                        //└─┘　└─┘
                        bool isI = true;
                        bool isJ = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isI = false;
                                isJ = false;
                                break;
                            case AgaruHiku.Agaru:
                                break;
                            case AgaruHiku.Hiku:
                                isI = false;
                                isJ = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isJ = false;
                                break;
                            case MigiHidari.Hidari:
                                isI = false;
                                break;
                            case MigiHidari.Sugu:
                                isI = false;
                                isJ = false;
                                break;
                        }

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isI) { srcAll.AddRange(KomanoKidou.SrcI_Kei(sengo, dst1)); }
                        if (isJ) { srcAll.AddRange(KomanoKidou.SrcJ_Kei(sengo, dst1)); }

                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion


                #region 銀
                //************************************************************
                // 銀
                //************************************************************
                case KomaSyurui.Gin:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┬─┬─┐
                        //│Ｈ│  │Ｂ│
                        //├─┼─┼─┤
                        //│　│至│  │先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        bool isB = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isH = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isB = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isH = false;
                                break;
                            case AgaruHiku.Agaru:
                                isB = false;
                                isH = false;
                                break;
                            case AgaruHiku.Hiku:
                                isD = false;
                                isE = false;
                                isF = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isE = false;
                                isF = false;
                                isH = false;
                                break;
                            case MigiHidari.Hidari:
                                isB = false;
                                isD = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isB = false;
                                isD = false;
                                isF = false;
                                isH = false;
                                break;
                        }

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Oh(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Oh(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Oh(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Oh(sengo, dst1)); }

                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region △金、△と金、△成香、△成桂、△成銀
                //************************************************************
                // △金、△と金、△成香、△成桂、△成銀
                //************************************************************
                case KomaSyurui.Kin:
                case KomaSyurui.Tokin:
                case KomaSyurui.NariKyo:
                case KomaSyurui.NariKei:
                case KomaSyurui.NariGin:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┬─┬─┐
                        //│  │Ａ│  │
                        //├─┼─┼─┤
                        //│Ｇ│至│Ｃ│先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        bool isA = true;
                        bool isC = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isG = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isA = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                break;
                            case AgaruHiku.Agaru:
                                isA = false;
                                isC = false;
                                isG = false;
                                break;
                            case AgaruHiku.Hiku:
                                isC = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isA = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                break;
                            case MigiHidari.Hidari:
                                isA = false;
                                isC = false;
                                isD = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isA = false;
                                isC = false;
                                isD = false;
                                isF = false;
                                isG = false;
                                break;
                        }

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Oh(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Oh(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Oh(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Oh(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Oh(sengo, dst1)); }

                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 王
                //************************************************************
                // 王
                //************************************************************
                case KomaSyurui.Oh:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //┌─┬─┬─┐
                        //│Ｈ│Ａ│Ｂ│
                        //├─┼─┼─┤
                        //│Ｇ│至│Ｃ│先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        bool isA = true;
                        bool isB = true;
                        bool isC = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isG = true;
                        bool isH = true;

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Oh(sengo, dst1)); }
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Oh(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Oh(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Oh(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Oh(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Oh(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Oh(sengo, dst1)); }

                        // 王は１つです。
                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 竜
                //************************************************************
                // 竜
                //************************************************************
                case KomaSyurui.Ryu:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │  │  │  │  │  │  │  │  │A7│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │A3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｈ│A0│Ｂ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │G7│G6│G5│G4│G3│G2│G1│G0│至│C0│C1│C2│C3│C4│C5│C6│C7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｆ│E0│Ｄ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │E4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E7│  │  │  │  │  │  │  │  │
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        bool isA = true;
                        bool isB = true;
                        bool isC = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isG = true;
                        bool isH = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isA = false;
                                isB = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isH = false;
                                break;
                            case AgaruHiku.Agaru:
                                isA = false;
                                isB = false;
                                isC = false;
                                isG = false;
                                isH = false;
                                break;
                            case AgaruHiku.Hiku:
                                isC = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isA = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                isH = false;
                                break;
                            case MigiHidari.Hidari:
                                isA = false;
                                isB = false;
                                isC = false;
                                isD = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isA = false;
                                isB = false;
                                isC = false;
                                isD = false;
                                isF = false;
                                isG = false;
                                isH = false;
                                break;
                        }

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Hisya(sengo, dst1)); }
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Oh(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Hisya(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Oh(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Hisya(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Oh(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Hisya(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Oh(sengo, dst1)); }

                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region 馬
                //************************************************************
                // 馬
                //************************************************************
                case KomaSyurui.Uma:
                    {
                        //----------
                        // 候補マス
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │H7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │B7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │H6│  │  │  │  │  │  │  │  │  │  │  │  │  │B6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │H5│  │  │  │  │  │  │  │  │  │  │  │B5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │H4│  │  │  │  │  │  │  │  │  │B4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │H3│  │  │  │  │  │  │  │B3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │H2│  │  │  │  │  │B2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │H1│  │  │  │B1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │H0│Ａ│B0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｇ│至│Ｃ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │F0│Ｅ│D0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │F1│  │  │  │D1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │F2│  │  │  │  │  │D2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │F3│  │  │  │  │  │  │  │D3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │F4│  │  │  │  │  │  │  │  │  │D4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │F5│  │  │  │  │  │  │  │  │  │  │  │D5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │F6│  │  │  │  │  │  │  │  │  │  │  │  │  │D6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │F7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │D7│
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        bool isA = true;
                        bool isB = true;
                        bool isC = true;
                        bool isD = true;
                        bool isE = true;
                        bool isF = true;
                        bool isG = true;
                        bool isH = true;

                        switch (agaruHiku)
                        {
                            case AgaruHiku.Yoru:
                                isA = false;
                                isB = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isH = false;
                                break;
                            case AgaruHiku.Agaru:
                                isA = false;
                                isB = false;
                                isC = false;
                                isG = false;
                                isH = false;
                                break;
                            case AgaruHiku.Hiku:
                                isC = false;
                                isD = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                break;
                        }

                        switch (migiHidari)
                        {
                            case MigiHidari.Migi:
                                isA = false;
                                isE = false;
                                isF = false;
                                isG = false;
                                isH = false;
                                break;
                            case MigiHidari.Hidari:
                                isA = false;
                                isB = false;
                                isC = false;
                                isD = false;
                                isE = false;
                                break;
                            case MigiHidari.Sugu:
                                isA = false;
                                isB = false;
                                isC = false;
                                isD = false;
                                isF = false;
                                isG = false;
                                isH = false;
                                break;
                        }

                        List<TeProcess> srcAll = new List<TeProcess>();
                        if (isA) { srcAll.AddRange(KomanoKidou.SrcA_Oh(sengo, dst1)); }
                        if (isB) { srcAll.AddRange(KomanoKidou.SrcB_Kaku(sengo, dst1)); }
                        if (isC) { srcAll.AddRange(KomanoKidou.SrcC_Oh(sengo, dst1)); }
                        if (isD) { srcAll.AddRange(KomanoKidou.SrcD_Kaku(sengo, dst1)); }
                        if (isE) { srcAll.AddRange(KomanoKidou.SrcE_Oh(sengo, dst1)); }
                        if (isF) { srcAll.AddRange(KomanoKidou.SrcF_Kaku(sengo, dst1)); }
                        if (isG) { srcAll.AddRange(KomanoKidou.SrcG_Oh(sengo, dst1)); }
                        if (isH) { srcAll.AddRange(KomanoKidou.SrcH_Kaku(sengo, dst1)); }

                        if (TuginoItte_JapanFugo.Hit(sengo, srcSyurui, srcAll, kyokumen, out hFoundKoma))
                        {
                            srcOkiba = Okiba.ShogiBan;
                            goto gt_EndSyurui;
                        }
                    }
                    break;
                #endregion

                #region エラー
                //************************************************************
                // エラー
                //************************************************************
                default:
                    {
                    }
                    break;
                #endregion

            }

        gt_EndShogiban:

            if (-1 == hFoundKoma && utsu)
            {
                // 駒台の駒を(明示的に)打ちます。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                List<int> hKomas = kyokumen.HKomas_ByOkibaSengoSyurui(srcOkiba, sengo, srcSyurui);

                if (0 < hKomas.Count)
                {
                    switch (sengo)
                    {
                        case Sengo.Gote:
                            srcOkiba = Okiba.Gote_Komadai;
                            break;
                        case Sengo.Sente:
                            srcOkiba = Okiba.Sente_Komadai;
                            break;
                        default:
                            srcOkiba = Okiba.CTRL_ERROR;
                            break;
                    }

                    hFoundKoma = hKomas[0];
                    goto gt_EndSyurui;
                }
            }

        gt_EndSyurui:


            int srcSuji;
            int srcDan;
            if (-1 != hFoundKoma)
            {
                // 将棋盤の上に駒がありました。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>


                srcSuji = kyokumen.KomaDoors[hFoundKoma].Suji;
                srcDan = kyokumen.KomaDoors[hFoundKoma].Dan;
            }
            else
            {
                // （符号に書かれていませんが）「打」のとき。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                switch (sengo)
                {
                    case Sengo.Gote:
                        srcOkiba = Okiba.Gote_Komadai;
                        break;
                    case Sengo.Sente:
                        srcOkiba = Okiba.Sente_Komadai;
                        break;
                    default:
                        srcOkiba = Okiba.CTRL_ERROR;
                        break;
                }

                srcSuji = TeProcess.CTRL_NOTHING_PROPERTY_SUJI;
                srcDan = TeProcess.CTRL_NOTHING_PROPERTY_DAN;
            }


            KomaSyurui syurui;
            if (NariFunari.Nari == nariFunari)
            {
                // 成ります
                syurui = Converter01.SyuruiToNariCase(srcSyurui);
            }
            else
            {
                // そのままです。
                syurui = srcSyurui;
            }


            // １手を、データにします。
            process = new TeProcess(
                sengo,
                Okiba.ShogiBan, //符号は将棋盤の升目です。
                srcOkiba,
                suji,
                dan,
                srcSuji,
                srcDan,
                syurui,
                srcSyurui,
                KomaSyurui.CTRL_NOTHING_PROPERTY // 取った駒の種類は分からない
            );
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 将棋盤上での検索
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="srcAll">候補マス</param>
        /// <param name="komas"></param>
        /// <returns></returns>
        private static bool Hit(
            Sengo sengo, KomaSyurui syurui, List<TeProcess> srcAll, Kyokumen kyokumen, out int hFoundKoma)
        {
            bool hit = false;
            hFoundKoma = -1;

            foreach (TeProcess masu in srcAll)//筋・段。（先後、種類は入っていません）
            {
                for (int h = 0; h < kyokumen.KomaDoors.Length; h++)//全駒
                {
                    TeProcess koma = kyokumen.KomaDoors[h];

                    if (sengo == koma.Sengo
                        && Okiba.ShogiBan == koma.Okiba
                        && syurui == koma.Syurui
                        && masu.MatchMasu(koma)
                        )
                    {
                        // 候補マスにいた
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                        hit = true;
                        hFoundKoma = h;
                        break;
                    }
                }
            }

            return hit;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// データを元に、符号１テキスト(*1)を作ります。
        /// ************************************************************************************************************************
        /// 
        ///         *1…「▲５五角右引」など。筋、段、同、駒種類、成、右左、上引、成、打。
        ///         
        /// 「移動した駒の種類」
        /// 「移動先の升番号」
        /// 「局面」
        /// 「棋譜」
        /// 
        /// があれば作れます。
        /// 
        /// ＜マウス操作で入力したときに呼び出されます＞
        /// </summary>
        /// <returns></returns>
        public static Fugo GetText_FromData(
            TeProcess process,//移動先、移動元、両方のマス番号
            Kyokumen kyokumen
            )
        {
            MigiHidari migiHidari;
            AgaruHiku agaruHiku;
            NariFunari nari;
            DaHyoji daHyoji = DaHyoji.No_Print; // “打”表示は、駒を打ったときとは異なります。


            switch (process.SrcSyurui)//「▲２二角成」なら、馬（dst）ではなくて角（src）。
            {
                #region 歩
                //************************************************************
                // 歩
                //************************************************************
                case KomaSyurui.Fu:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //┌─┬─┬─┐
                        //│  │  │  │
                        //├─┼─┼─┤
                        //│  │至│  │
                        //├─┼─┼─┤
                        //│  │Ｅ│  │
                        //└─┴─┴─┘
                        List<TeProcess> srcE = KomanoKidou.SrcE_Oh(process.Sengo, process);
                        TeProcess src = process.Src();

                        //----------
                        // 棋譜の現局面：競合駒
                        //----------
                        List<int> kmE = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcE);

                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            // Ｅにいた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            // 歩に右左、引上はありません。
                            migiHidari = MigiHidari.No_Print;
                            agaruHiku = AgaruHiku.No_Print;
                        }
                        else
                        {
                            // どこからか飛んできた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            // 歩に右左、引上はありません。
                            migiHidari = MigiHidari.No_Print;
                            agaruHiku = AgaruHiku.No_Print;
                        }

                        // 「打」解除： 競合範囲全てに競合駒がなければ。
                        if (daHyoji == DaHyoji.Visible && process.NeverOnaji(kyokumen, kmE)) { daHyoji = DaHyoji.No_Print; }

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.CTRL_SONOMAMA;
                        }
                    }
                    break;
                #endregion

                #region 飛
                //************************************************************
                // 飛
                //************************************************************
                case KomaSyurui.Hisya:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │  │  │  │  │  │  │  │  │A7│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │A3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A0│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │G7│G6│G5│G4│G3│G2│G1│G0│至│C0│C1│C2│C3│C4│C5│C6│C7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E0│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │E4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E7│  │  │  │  │  │  │  │  │
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        List<TeProcess> srcA = KomanoKidou.SrcA_Hisya(process.Sengo, process);
                        List<TeProcess> srcC = KomanoKidou.SrcC_Hisya(process.Sengo, process);
                        List<TeProcess> srcE = KomanoKidou.SrcE_Hisya(process.Sengo, process);
                        List<TeProcess> srcG = KomanoKidou.SrcG_Hisya(process.Sengo, process);
                        TeProcess src = process.Src();

                        //----------
                        // 棋譜の現局面：競合駒
                        //----------
                        List<int> kmA = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcA);
                        List<int> kmC = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcC);
                        List<int> kmE = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcE);
                        List<int> kmG = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcG);

                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcA))
                        {
                            //----------
                            // Ａにいた
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.No_Print;
                        }
                        else if (src.ExistsIn(srcC))
                        {
                            //----------
                            // Ｃにいた
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            //----------
                            // Ｅにいた
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.No_Print;
                        }
                        else if (src.ExistsIn(srcG))
                        {
                            //----------
                            // Ｇにいた
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                        }

                        // 「右」解除： Ｇに競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kyokumen, kmG)) { migiHidari = MigiHidari.No_Print; }

                        // 「左」解除： Ｃに競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kyokumen, kmC)) { migiHidari = MigiHidari.No_Print; }

                        // 「上」解除： Ａ、Ｃ、Ｇに競合駒がなければ。
                        //if (agaruHiku == AgaruHiku.Agaru && process.NeverOnaji(kmA) && process.NeverOnaji(kmC) && process.NeverOnaji(kmG)) { agaruHiku = AgaruHiku.None; }
                        if (agaruHiku == AgaruHiku.Agaru && process.NeverOnaji(kyokumen, kmA, kmC, kmG)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「引」解除： Ｃ、Ｅ、Ｇに競合駒がなければ。
                        //if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kmC) && process.NeverOnaji(kmE) && process.NeverOnaji(kmG)) { agaruHiku = AgaruHiku.None; }
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kyokumen, kmC, kmE, kmG)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「寄」解除： Ａ、Ｅに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Yoru && process.NeverOnaji(kyokumen, kmA) && process.NeverOnaji(kyokumen, kmE)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「打」解除： 競合範囲全てに競合駒がなければ。
                        if (daHyoji == DaHyoji.Visible && process.NeverOnaji(kyokumen, kmA, kmC, kmE, kmG)) { daHyoji = DaHyoji.No_Print; }

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.CTRL_SONOMAMA;
                        }
                    }
                    break;
                #endregion

                #region 角
                //************************************************************
                // 角
                //************************************************************
                case KomaSyurui.Kaku:
                    {
                        //----------
                        // 競合駒マス(range masu)
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │H7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │B7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │H6│  │  │  │  │  │  │  │  │  │  │  │  │  │B6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │H5│  │  │  │  │  │  │  │  │  │  │  │B5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │H4│  │  │  │  │  │  │  │  │  │B4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │H3│  │  │  │  │  │  │  │B3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │H2│  │  │  │  │  │B2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │H1│  │  │  │B1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │H0│  │B0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │至│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │F0│  │D0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │F1│  │  │  │D1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │F2│  │  │  │  │  │D2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │F3│  │  │  │  │  │  │  │D3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │F4│  │  │  │  │  │  │  │  │  │D4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │F5│  │  │  │  │  │  │  │  │  │  │  │D5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │F6│  │  │  │  │  │  │  │  │  │  │  │  │  │D6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │F7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │D7│
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        List<TeProcess> srcB = KomanoKidou.SrcB_Kaku(process.Sengo, process);
                        List<TeProcess> srcD = KomanoKidou.SrcD_Kaku(process.Sengo, process);
                        List<TeProcess> srcF = KomanoKidou.SrcF_Kaku(process.Sengo, process);
                        List<TeProcess> srcH = KomanoKidou.SrcH_Kaku(process.Sengo, process);
                        TeProcess src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        List<int> kmB = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcB);
                        List<int> kmD = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcD);
                        List<int> kmF = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcF);
                        List<int> kmH = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcH);


                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcB))
                        {
                            //----------
                            // 移動前はＢだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcD))
                        {
                            //----------
                            // 移動前はＤだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcF))
                        {
                            //----------
                            // 移動前はＦだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else if (src.ExistsIn(srcH))
                        {
                            //----------
                            // 移動前はＨだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                        }

                        // 「右」解除： Ｆ、Ｈに競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kyokumen, kmF, kmH)) { migiHidari = MigiHidari.No_Print; }

                        // 「左」解除： Ｂ、Ｄに競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kyokumen, kmB, kmD)) { migiHidari = MigiHidari.No_Print; }

                        // 「上」解除： Ｂ、Ｈに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Agaru && process.NeverOnaji(kyokumen, kmB, kmH)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「引」解除： Ｄ、Ｆに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kyokumen, kmD, kmF)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「寄」解除： 角は寄れません。

                        // 「打」解除： 競合範囲全てに競合駒がなければ。
                        if (daHyoji == DaHyoji.Visible && process.NeverOnaji(kyokumen, kmB, kmD, kmF, kmH)) { daHyoji = DaHyoji.No_Print; }

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.CTRL_SONOMAMA;
                        }
                    }
                    break;
                #endregion

                #region 香
                //************************************************************
                // 香
                //************************************************************
                case KomaSyurui.Kyo:
                    {
                        //  ┌─┬─┬─┐
                        //  │  │至│  │
                        //  ├─┼─┼─┤
                        //  │  │E0│  │
                        //  ├─┼─┼─┤
                        //  │  │E1│  │
                        //  ├─┼─┼─┤
                        //  │  │E2│  │
                        //  ├─┼─┼─┤
                        //  │  │E3│  │
                        //  ├─┼─┼─┤
                        //  │  │E4│  │
                        //  ├─┼─┼─┤
                        //  │  │E5│  │
                        //  ├─┼─┼─┤
                        //  │  │E6│  │
                        //  ├─┼─┼─┤
                        //  │  │E7│  │
                        //  └─┴─┴─┘
                        List<TeProcess> srcE = KomanoKidou.SrcE_Hisya(process.Sengo, process);
                        TeProcess src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        List<int> kmE = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcE);

                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            //----------
                            // 移動前はＥだった
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                        }

                        // 「右」解除： 香に右はありません。
                        // 「左」解除： 香に左はありません。
                        // 「上」解除： 香に上はありません。
                        // 「引」解除： 香に引はありません。
                        // 「寄」解除： 香に寄はありません。

                        // 「打」解除： 競合範囲全てに競合駒がなければ。
                        if (daHyoji == DaHyoji.Visible && process.NeverOnaji(kyokumen, kmE)) { daHyoji = DaHyoji.No_Print; }

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.CTRL_SONOMAMA;
                        }
                    }
                    break;
                #endregion

                #region 桂
                //************************************************************
                // 桂
                //************************************************************
                case KomaSyurui.Kei:
                    {
                        //----------
                        // 競合駒マス
                        //----------
                        //┌─┐　┌─┐
                        //│  │　│  │
                        //├─┼─┼─┤
                        //│　│  │  │
                        //├─┼─┼─┤
                        //│　│至│  │先手から見た図
                        //├─┼─┼─┤
                        //│　│  │  │
                        //├─┼─┼─┤
                        //│Ｊ│　│Ｉ│
                        //└─┘　└─┘
                        List<TeProcess> srcI = KomanoKidou.SrcI_Kei(process.Sengo, process);
                        List<TeProcess> srcJ = KomanoKidou.SrcJ_Kei(process.Sengo, process);
                        TeProcess src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        List<int> kmI = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcI);
                        List<int> kmJ = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcJ);

                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcI))
                        {
                            //----------
                            // 移動前はＩだった
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcJ))
                        {
                            //----------
                            // 移動前はＪだった
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                        }

                        // 「右」解除： Ｊに競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kyokumen, kmJ)) { migiHidari = MigiHidari.No_Print; }

                        // 「左」解除： Ｉに競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kyokumen, kmI)) { migiHidari = MigiHidari.No_Print; }

                        // 「打」解除： 競合範囲全てに競合駒がなければ。
                        if (daHyoji == DaHyoji.Visible && process.NeverOnaji(kyokumen, kmI, kmJ)) { daHyoji = DaHyoji.No_Print; }

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.CTRL_SONOMAMA;
                        }
                    }
                    break;
                #endregion

                #region 銀
                //************************************************************
                // 銀
                //************************************************************
                case KomaSyurui.Gin:
                    {
                        //----------
                        // 競合駒マス(range masu)
                        //----------
                        //┌─┬─┬─┐
                        //│Ｈ│  │Ｂ│
                        //├─┼─┼─┤
                        //│　│至│  │先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        List<TeProcess> srcB = KomanoKidou.SrcB_Oh(process.Sengo, process);
                        List<TeProcess> srcD = KomanoKidou.SrcD_Oh(process.Sengo, process);
                        List<TeProcess> srcE = KomanoKidou.SrcE_Oh(process.Sengo, process);
                        List<TeProcess> srcF = KomanoKidou.SrcF_Oh(process.Sengo, process);
                        List<TeProcess> srcH = KomanoKidou.SrcH_Oh(process.Sengo, process);
                        TeProcess src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        List<int> kmB = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcB);
                        List<int> kmD = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcD);
                        List<int> kmE = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcE);
                        List<int> kmF = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcF);
                        List<int> kmH = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcH);

                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcB))
                        {
                            // 移動前はＢだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcD))
                        {
                            // 移動前はＤだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            // 移動前はＥだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Sugu;
                        }
                        else if (src.ExistsIn(srcF))
                        {
                            // 移動前はＦだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else if (src.ExistsIn(srcH))
                        {
                            // 移動前はＨだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                        }

                        // 「右」解除： ①Ｅ、Ｆ、Ｈのどこにも競合駒がなければ。
                        //              ②Ｈに競合駒がなく引があるなら。
                        //              ③Ｅ、Ｆに競合駒がなく上があるなら。
                        if (migiHidari == MigiHidari.Migi && (
                            process.NeverOnaji(kyokumen, kmE, kmF, kmH)
                            || (process.NeverOnaji(kyokumen, kmH) && agaruHiku == AgaruHiku.Hiku)
                            || (process.NeverOnaji(kyokumen, kmE, kmF) && agaruHiku == AgaruHiku.Agaru)
                            )) { migiHidari = MigiHidari.No_Print; }

                        // 「左」解除： ①Ｂ、Ｄ、Ｅのどこにも競合駒がなければ。
                        //              ②Ｂに競合駒がなく引があるなら。
                        //              ③Ｄ、Ｅに競合駒がなく上があるなら。
                        if (migiHidari == MigiHidari.Hidari && (
                            process.NeverOnaji(kyokumen, kmB, kmD, kmE)
                            || (process.NeverOnaji(kyokumen, kmB) && agaruHiku == AgaruHiku.Hiku)
                            || (process.NeverOnaji(kyokumen, kmD, kmE) && agaruHiku == AgaruHiku.Agaru)
                            )) { migiHidari = MigiHidari.No_Print; }

                        // 「直」解除： Ｄ、Ｆのどちらにも競合駒がなければ。
                        if (migiHidari == MigiHidari.Sugu && process.NeverOnaji(kyokumen, kmD, kmF)) { migiHidari = MigiHidari.No_Print; }

                        // 「上」解除： Ｂ、Ｈのどこにも競合駒がなければ。また、直があるなら。
                        if (agaruHiku == AgaruHiku.Agaru && (process.NeverOnaji(kyokumen, kmB, kmH) || migiHidari == MigiHidari.Sugu)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「引」解除： ①Ｂ、Ｄ、Ｅ、Ｆのどこにも競合駒がなければ。
                        //              ②Ｄ、Ｅ、Ｆ、Ｈのどこにも競合駒がなければ。
                        //              ③Ｄに競合駒がなく、右があるなら。
                        //              ④Ｆに競合駒がなく、左があるなら。
                        if (agaruHiku == AgaruHiku.Hiku &&
                            (
                            process.NeverOnaji(kyokumen, kmB, kmD, kmE, kmF)
                            || process.NeverOnaji(kyokumen, kmD, kmE, kmF, kmH)
                            || (process.NeverOnaji(kyokumen, kmD) && migiHidari == MigiHidari.Migi)
                            || (process.NeverOnaji(kyokumen, kmF) && migiHidari == MigiHidari.Hidari)
                            )) { agaruHiku = AgaruHiku.No_Print; }

                        // 「寄」解除： 銀は寄れません。

                        // 「打」解除： 競合範囲全てに競合駒がなければ。
                        if (daHyoji == DaHyoji.Visible && process.NeverOnaji(kyokumen, kmB, kmD, kmE, kmF, kmH)) { daHyoji = DaHyoji.No_Print; }

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.CTRL_SONOMAMA;
                        }
                    }
                    break;
                #endregion

                #region △金、△と金、△成香、△成桂、△成銀
                //************************************************************
                // △金、△と金、△成香、△成桂、△成銀
                //************************************************************
                case KomaSyurui.Kin:
                case KomaSyurui.Tokin:
                case KomaSyurui.NariKyo:
                case KomaSyurui.NariKei:
                case KomaSyurui.NariGin:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //┌─┬─┬─┐
                        //│  │Ａ│  │
                        //├─┼─┼─┤
                        //│Ｇ│至│Ｃ│先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        List<TeProcess> srcA = KomanoKidou.SrcA_Oh(process.Sengo, process);
                        List<TeProcess> srcC = KomanoKidou.SrcC_Oh(process.Sengo, process);
                        List<TeProcess> srcD = KomanoKidou.SrcD_Oh(process.Sengo, process);
                        List<TeProcess> srcE = KomanoKidou.SrcE_Oh(process.Sengo, process);
                        List<TeProcess> srcF = KomanoKidou.SrcF_Oh(process.Sengo, process);
                        List<TeProcess> srcG = KomanoKidou.SrcG_Oh(process.Sengo, process);
                        TeProcess src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        List<int> kmA = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcA);
                        List<int> kmC = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcC);
                        List<int> kmD = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcD);
                        List<int> kmE = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcE);
                        List<int> kmF = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcF);
                        List<int> kmG = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcG);


                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcA))
                        {
                            //----------
                            // 移動前はＡだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.No_Print;
                            //// 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcC))
                        {
                            //----------
                            // 移動前はＣだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Migi;
                            //// 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcF))
                        {
                            //----------
                            // 移動前はＤだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                            //// 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            //----------
                            // 移動前はＥだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Sugu;
                            //// 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcD))
                        {
                            //----------
                            // 移動前はＦだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                            //// 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcG))
                        {
                            //----------
                            // 移動前はＧだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Hidari;
                            //// 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                        }


                        // 「右」解除： ①Ｅ、Ｆ、Ｇのどちらにも競合駒がなければ。
                        //              ②Ｇに競合駒がなく、寄があるなら。
                        //              ③上があり、Ｅ、Ｆのどちらにも競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && (
                            process.NeverOnaji(kyokumen, kmE, kmF, kmG)
                            || (process.NeverOnaji(kyokumen, kmG) && agaruHiku == AgaruHiku.Yoru)
                            || (AgaruHiku.Agaru == agaruHiku && process.NeverOnaji(kyokumen, kmE, kmF))
                            )) { migiHidari = MigiHidari.No_Print; }

                        // 「左」解除： ①Ｃ、Ｄ、Ｅのどちらにも競合駒がなければ。
                        //              ②Ｃに競合駒がなく、寄があるなら。
                        //              ③上があり、Ｄ、Ｅのどちらにも競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && (
                            process.NeverOnaji(kyokumen, kmC, kmD, kmE)
                            || (process.NeverOnaji(kyokumen, kmC) && agaruHiku == AgaruHiku.Yoru)
                            || (AgaruHiku.Agaru == agaruHiku && process.NeverOnaji(kyokumen, kmD, kmE))
                            )) { migiHidari = MigiHidari.No_Print; }

                        // 「直」解除： Ｄ、Ｆのどちらにも競合駒がなければ。
                        if (migiHidari == MigiHidari.Sugu && process.NeverOnaji(kyokumen, kmD, kmF)) { migiHidari = MigiHidari.No_Print; }

                        // 「上」解除： ①Ａ、Ｃ、Ｇのどこにも競合駒がなければ。
                        //              ②直があるなら。
                        //              ③Ｃに競合駒がなく、右があるなら。
                        //              ④Ｇに競合駒がなく、左があるなら。
                        if (agaruHiku == AgaruHiku.Agaru &&
                            (
                            process.NeverOnaji(kyokumen, kmA, kmC, kmG)
                            || migiHidari == MigiHidari.Sugu
                            || process.NeverOnaji(kyokumen, kmC) && migiHidari == MigiHidari.Migi
                            || process.NeverOnaji(kyokumen, kmG) && migiHidari == MigiHidari.Hidari
                            )
                            ) { agaruHiku = AgaruHiku.No_Print; }

                        // 「引」解除： Ｃ、Ｄ、Ｅ、Ｆ、Ｇのどこにも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kyokumen, kmC, kmD, kmE, kmF, kmG)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「寄」解除： ①Ａ、Ｄ、Ｅ、Ｆのどこにも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Yoru && process.NeverOnaji(kyokumen, kmA, kmD, kmE, kmF)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「打」解除： 競合範囲全てに競合駒がなければ。
                        if (daHyoji == DaHyoji.Visible && process.NeverOnaji(kyokumen, kmA, kmC, kmD, kmE, kmF, kmG)) { daHyoji = DaHyoji.No_Print; }

                        //----------
                        // 成れません
                        //----------
                        nari = NariFunari.CTRL_SONOMAMA;
                    }
                    break;
                #endregion

                #region 王
                //************************************************************
                // 王
                //************************************************************
                case KomaSyurui.Oh:
                    {
                        //----------
                        // 競合駒マス(range masu)
                        //----------
                        //┌─┬─┬─┐
                        //│Ｈ│Ａ│Ｂ│
                        //├─┼─┼─┤
                        //│Ｇ│至│Ｃ│先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘

                        migiHidari = MigiHidari.No_Print;
                        agaruHiku = AgaruHiku.No_Print;

                        //----------
                        // 王は成れません
                        //----------
                        nari = NariFunari.CTRL_SONOMAMA;
                    }
                    break;
                #endregion

                #region 竜
                //************************************************************
                // 竜
                //************************************************************
                case KomaSyurui.Ryu:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │  │  │  │  │  │  │  │  │A7│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │A3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｈ│A0│Ｂ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │G7│G6│G5│G4│G3│G2│G1│G0│至│C0│C1│C2│C3│C4│C5│C6│C7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｆ│E0│Ｄ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │E4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E7│  │  │  │  │  │  │  │  │
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        List<TeProcess> srcA = KomanoKidou.SrcA_Hisya(process.Sengo, process);
                        List<TeProcess> srcB = KomanoKidou.SrcB_Oh(process.Sengo, process);
                        List<TeProcess> srcC = KomanoKidou.SrcC_Hisya(process.Sengo, process);
                        List<TeProcess> srcD = KomanoKidou.SrcD_Oh(process.Sengo, process);
                        List<TeProcess> srcE = KomanoKidou.SrcE_Hisya(process.Sengo, process);
                        List<TeProcess> srcF = KomanoKidou.SrcF_Oh(process.Sengo, process);
                        List<TeProcess> srcG = KomanoKidou.SrcG_Hisya(process.Sengo, process);
                        List<TeProcess> srcH = KomanoKidou.SrcH_Oh(process.Sengo, process);
                        TeProcess src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        List<int> kmA = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcA);
                        List<int> kmB = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcB);
                        List<int> kmC = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcC);
                        List<int> kmD = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcD);
                        List<int> kmE = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcE);
                        List<int> kmF = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcF);
                        List<int> kmG = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcG);
                        List<int> kmH = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcH);


                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcA))
                        {
                            //----------
                            // 移動前はＡだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.No_Print;
                        }
                        else if (src.ExistsIn(srcB))
                        {
                            //----------
                            // 移動前はＢだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcC))
                        {
                            //----------
                            // 移動前はＣだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcD))
                        {
                            //----------
                            // 移動前はＤだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            //----------
                            // 移動前はＥだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.No_Print;
                        }
                        else if (src.ExistsIn(srcF))
                        {
                            //----------
                            // 移動前はＦだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else if (src.ExistsIn(srcG))
                        {
                            //----------
                            // 移動前はＧだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else if (src.ExistsIn(srcH))
                        {
                            //----------
                            // 移動前はＨだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                        }

                        // 「右」解除： Ａ、Ｅ、Ｆ、Ｇ、Ｈに１つも競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kyokumen, kmA, kmE, kmF, kmG, kmH)) { migiHidari = MigiHidari.No_Print; }

                        // 「左」解除： Ａ、Ｂ、Ｃ、Ｄ、Ｅに１つも競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kyokumen, kmA, kmB, kmC, kmD, kmE)) { migiHidari = MigiHidari.No_Print; }

                        // 「上」解除： Ａ、Ｂ、Ｃ、Ｇ、Ｈに１つも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Agaru && process.NeverOnaji(kyokumen, kmA, kmB, kmC, kmG, kmH)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「引」解除： Ｃ、Ｄ、Ｅ、Ｆ、Ｇに１つも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kyokumen, kmC, kmD, kmE, kmF, kmG)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「寄」解除： Ａ、Ｂ、Ｄ、Ｅ、Ｆ、Ｈに１つも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Yoru && process.NeverOnaji(kyokumen, kmA, kmB, kmD, kmE, kmF, kmH)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「打」解除： 競合範囲全てに競合駒がなければ。
                        if (daHyoji == DaHyoji.Visible && process.NeverOnaji(kyokumen, kmA, kmB, kmC, kmD, kmE, kmF, kmG, kmH)) { daHyoji = DaHyoji.No_Print; }

                        //----------
                        // 成れません
                        //----------
                        nari = NariFunari.CTRL_SONOMAMA;
                    }
                    break;
                #endregion

                #region 馬
                //************************************************************
                // 馬
                //************************************************************
                case KomaSyurui.Uma:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │H7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │B7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │H6│  │  │  │  │  │  │  │  │  │  │  │  │  │B6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │H5│  │  │  │  │  │  │  │  │  │  │  │B5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │H4│  │  │  │  │  │  │  │  │  │B4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │H3│  │  │  │  │  │  │  │B3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │H2│  │  │  │  │  │B2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │H1│  │  │  │B1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │H0│Ａ│B0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｇ│至│Ｃ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │F0│Ｅ│D0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │F1│  │  │  │D1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │F2│  │  │  │  │  │D2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │F3│  │  │  │  │  │  │  │D3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │F4│  │  │  │  │  │  │  │  │  │D4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │F5│  │  │  │  │  │  │  │  │  │  │  │D5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │F6│  │  │  │  │  │  │  │  │  │  │  │  │  │D6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │F7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │D7│
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        List<TeProcess> srcA = KomanoKidou.SrcA_Oh(process.Sengo, process);
                        List<TeProcess> srcB = KomanoKidou.SrcB_Kaku(process.Sengo, process);
                        List<TeProcess> srcC = KomanoKidou.SrcC_Oh(process.Sengo, process);
                        List<TeProcess> srcD = KomanoKidou.SrcD_Kaku(process.Sengo, process);
                        List<TeProcess> srcE = KomanoKidou.SrcE_Oh(process.Sengo, process);
                        List<TeProcess> srcF = KomanoKidou.SrcF_Kaku(process.Sengo, process);
                        List<TeProcess> srcG = KomanoKidou.SrcG_Oh(process.Sengo, process);
                        List<TeProcess> srcH = KomanoKidou.SrcH_Kaku(process.Sengo, process);
                        TeProcess src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        List<int> kmA = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcA);
                        List<int> kmB = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcB);
                        List<int> kmC = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcC);
                        List<int> kmD = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcD);
                        List<int> kmE = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcE);
                        List<int> kmF = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcF);
                        List<int> kmG = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcG);
                        List<int> kmH = kyokumen.KomaHandles_EachSrc(process.Sengo, process, srcH);


                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcB))
                        {
                            //----------
                            // 移動前はＢだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcD))
                        {
                            //----------
                            // 移動前はＤだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcF))
                        {
                            //----------
                            // 移動前はＦだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else if (src.ExistsIn(srcH))
                        {
                            //----------
                            // 移動前はＨだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else if (src.ExistsIn(srcA))
                        {
                            //----------
                            // 移動前はＡだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.No_Print;
                        }
                        else if (src.ExistsIn(srcC))
                        {
                            //----------
                            // 移動前はＣだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Migi;
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            //----------
                            // 移動前はＥだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.No_Print;
                        }
                        else if (src.ExistsIn(srcG))
                        {
                            //----------
                            // 移動前はＧだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Hidari;
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.No_Print;
                            migiHidari = MigiHidari.No_Print;
                        }

                        // 「右」解除： Ａ、Ｅ、Ｆ、Ｇ、Ｈに競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kyokumen, kmA, kmE, kmF, kmG, kmH)) { migiHidari = MigiHidari.No_Print; }

                        // 「左」解除： Ａ、Ｂ、Ｃ、Ｄ、Ｅに競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kyokumen, kmA, kmB, kmC, kmD, kmE)) { migiHidari = MigiHidari.No_Print; }

                        // 「上」解除： Ａ、Ｂ、Ｃ、Ｇ、Ｈに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Agaru && process.NeverOnaji(kyokumen, kmA, kmB, kmC, kmG, kmH)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「引」解除： Ｃ、Ｄ、Ｅ、Ｆ、Ｇに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kyokumen, kmC, kmD, kmE, kmF, kmG)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「寄」解除： Ａ、Ｂ、Ｄ、Ｅ、Ｆ、Ｈに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Yoru && process.NeverOnaji(kyokumen, kmA, kmB, kmD, kmE, kmF, kmH)) { agaruHiku = AgaruHiku.No_Print; }

                        // 「打」解除： 競合範囲全てに競合駒がなければ。
                        if (daHyoji == DaHyoji.Visible && process.NeverOnaji(kyokumen, kmA, kmB, kmC, kmD, kmE, kmF, kmG, kmH)) { daHyoji = DaHyoji.No_Print; }

                        //----------
                        // 成れません
                        //----------
                        nari = NariFunari.CTRL_SONOMAMA;
                    }
                    break;
                #endregion

                #region エラー
                //************************************************************
                // エラー
                //************************************************************
                default:
                    {
                        if (Sengo.Gote == process.Sengo)
                        {
                            //******************************
                            // △後手
                            //******************************
                        }
                        else
                        {
                            //******************************
                            // ▲先手
                            //******************************
                        }

                        migiHidari = MigiHidari.No_Print;
                        agaruHiku = AgaruHiku.No_Print;

                        //----------
                        // TODO: 移動前の駒が成る前かどうか
                        //----------
                        nari = NariFunari.CTRL_SONOMAMA;
                    }
                    break;
                #endregion

            }

            Fugo fugo = new Fugo(
                process.SrcSyurui,//「▲２二角成」のとき、dstだと馬になってしまう。srcの角を使う。
                migiHidari,
                agaruHiku,
                nari,
                daHyoji
                );

            return fugo;
        }

    }


}
