﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Reflection;
using System.Text.RegularExpressions;

using Xenon.KifuNarabe.L01_Communication;
using Xenon.KifuNarabe.L02_Common;

namespace Xenon.KifuNarabe.L03_TuginoItte
{

    /// <summary>
    /// ************************************************************************************************************************
    /// 「7g7f」といった記述を、棋譜データの持ち方に変換します。
    /// ************************************************************************************************************************
    /// </summary>
    public abstract class TuginoItte_Sfen
    {
        /// <summary>
        /// ************************************************************************************************************************
        /// 「lnsgkgsn1/1r5b1/ppppppppp/9/9/9/PPPPPPPPP/1B5R1/LNSGKGSNL w - 1」といった記述を解析します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <returns></returns>
        public static bool GetDataStartpos_FromText(string text, out string restText, out SfenStartpos sfenStartpos)
        {
            System.Console.WriteLine("TranslateSfenStartpos: text=" + text);

            bool successful = false;
            sfenStartpos = null;

            restText = text;

            //------------------------------------------------------------
            // リスト作成
            //------------------------------------------------------------
            Regex regex = new Regex(
                @"^\s*" +
                @"([123456789KRBGSNLPkrbgsnlp\+]+)/" +//1段目
                @"([123456789KRBGSNLPkrbgsnlp\+]+)/" +//2段目
                @"([123456789KRBGSNLPkrbgsnlp\+]+)/" +//3段目
                @"([123456789KRBGSNLPkrbgsnlp\+]+)/" +//4段目
                @"([123456789KRBGSNLPkrbgsnlp\+]+)/" +//5段目
                @"([123456789KRBGSNLPkrbgsnlp\+]+)/" +//6段目
                @"([123456789KRBGSNLPkrbgsnlp\+]+)/" +//7段目
                @"([123456789KRBGSNLPkrbgsnlp\+]+)/" +//8段目
                @"([123456789KRBGSNLPkrbgsnlp\+]+) " +//9段目
                @"(b|w) " +//先後
                @"\-?" +
                @"(?:K(\d+))?" +//▲王
                @"(?:R(\d+))?" +//▲飛
                @"(?:B(\d+))?" +//▲角
                @"(?:G(\d+))?" +//▲金
                @"(?:S(\d+))?" +//▲銀
                @"(?:N(\d+))?" +//▲桂
                @"(?:L(\d+))?" +//▲香
                @"(?:P(\d+))?" +//▲歩
                @"(?:k(\d+))?" +//△王
                @"(?:r(\d+))?" +//△飛
                @"(?:b(\d+))?" +//△角
                @"(?:g(\d+))?" +//△金
                @"(?:s(\d+))?" +//△銀
                @"(?:n(\d+))?" +//△桂
                @"(?:l(\d+))?" +//△香
                @"(?:p(\d+))?" +//△歩
                @" (\d+)" +//手目
                @"",
                RegexOptions.Singleline
            );

            MatchCollection mc = regex.Matches(text);
            foreach (Match m in mc)
            {
                if (0 < m.Groups.Count)
                {
                    successful = true;

                    // 残りのテキスト
                    restText = text.Substring(0, m.Index) + text.Substring(m.Index + m.Length, text.Length - (m.Index + m.Length));

                    sfenStartpos = new SfenStartpos(
                        m.Groups[1].Value,  //1段目
                        m.Groups[2].Value,  //2段目
                        m.Groups[3].Value,  //3段目
                        m.Groups[4].Value,  //4段目
                        m.Groups[5].Value,  //5段目
                        m.Groups[6].Value,  //6段目
                        m.Groups[7].Value,  //7段目
                        m.Groups[8].Value,  //8段目
                        m.Groups[9].Value,  //9段目
                        m.Groups[10].Value,  //先後
                        m.Groups[11].Value,  //▲王
                        m.Groups[12].Value,  //▲飛
                        m.Groups[13].Value,  //▲角
                        m.Groups[14].Value,  //▲金
                        m.Groups[15].Value,  //▲銀
                        m.Groups[16].Value,  //▲桂
                        m.Groups[17].Value,  //▲香
                        m.Groups[18].Value,  //▲歩
                        m.Groups[19].Value,  //△王
                        m.Groups[20].Value,  //△飛
                        m.Groups[21].Value,  //△角
                        m.Groups[22].Value,  //△金
                        m.Groups[23].Value,  //△銀
                        m.Groups[24].Value,  //△桂
                        m.Groups[25].Value,  //△香
                        m.Groups[26].Value,  //△歩
                        m.Groups[27].Value  //手目
                        );
                }

                // 最初の１件だけ処理して終わります。
                break;
            }

            restText = restText.Trim();

            return successful;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// テキスト形式の符号「7g7f 3c3d 6g6f…」の最初の要素を、切り取ってプロセスに変換します。
        /// ************************************************************************************************************************
        /// 
        /// 再生、コマ送りで利用。
        /// </summary>
        /// <returns></returns>
        public static bool GetData_FromText(string text, out string restText, out TeProcess process, Kifu kifu)
        {
            process = null;

            bool successful = false;

            restText = text;

            //------------------------------------------------------------
            // リスト作成
            //------------------------------------------------------------
            Regex regex = new Regex(
                @"^\s*([123456789PLNSGKRB])([abcdefghi\*])([123456789])([abcdefghi])(\+)?",
                RegexOptions.Singleline
            );

            MatchCollection mc = regex.Matches(text);
            foreach (Match m in mc)
            {
                if (0 < m.Groups.Count)
                {
                    successful = true;

                    // 残りのテキスト
                    restText = text.Substring(0, m.Index) + text.Substring(m.Index + m.Length, text.Length - (m.Index + m.Length));

                    TuginoItte_Sfen.GetData_FromTextSub(
                        m.Groups[1].Value,  //123456789 か、 PLNSGKRB
                        m.Groups[2].Value,  //abcdefghi か、 *
                        m.Groups[3].Value,  //123456789
                        m.Groups[4].Value,  //abcdefghi
                        m.Groups[5].Value,  //+
                        out process,
                        kifu
                        );
                }

                // 最初の１件だけ処理して終わります。
                break;
            }

            restText = restText.Trim();

            return successful;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 符号１「7g7f」を元に、process を作ります。
        /// ************************************************************************************************************************
        /// 
        /// ＜再生、コマ送りで呼び出されます＞
        /// </summary>
        /// <returns></returns>
        private static void GetData_FromTextSub(
            string str1, //123456789 か、 PLNSGKRB
            string str2, //abcdefghi か、 *
            string str3, //123456789
            string str4, //abcdefghi
            string strNari, //+
            out TeProcess process,
            Kifu kifu
            )
        {
            //------------------------------
            // 1
            //------------------------------
            KomaSyurui uttaSyurui; // 打った駒の種類
            Converter01.SfenUttaSyurui(str1, out uttaSyurui);

            int srcSuji = TeProcess.CTRL_NOTHING_PROPERTY_SUJI;
            if (KomaSyurui.CTRL_NOTHING_PROPERTY == uttaSyurui)
            {
                if (!int.TryParse(str1, out srcSuji))
                {
                }
            }
            else
            {
            }

            //------------------------------
            // 2
            //------------------------------
            int srcDan = TeProcess.CTRL_NOTHING_PROPERTY_DAN;
            if ("*" == str2)
            {
                // 「打」でした。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
            }
            else
            {
                srcDan = Converter01.AlphabetToInt(str2);
            }

            //------------------------------
            // 3
            //------------------------------
            int suji;
            if(!int.TryParse(str3, out suji))
            {
            }

            //------------------------------
            // 4
            //------------------------------
            int dan;
            dan = Converter01.AlphabetToInt(str4);

            

            int hKoma = kifu.Kyokumen.HKoma_AtMasuDokodemo(Okiba.ShogiBan, new TeProcess(
                Sengo.CTRL_NOTHING_PROPERTY,
                Okiba.CTRL_NOTHING_PROPERTY,
                srcSuji,
                srcDan,
                KomaSyurui.CTRL_NOTHING_PROPERTY
                ));

            KomaSyurui syurui;
            KomaSyurui srcSyurui;
            Okiba srcOkiba;

            if (-1 != hKoma)
            {
                // 盤上の駒を指した場合
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                syurui = kifu.Kyokumen.KomaDoors[hKoma].Syurui;
                srcSyurui = kifu.Kyokumen.KomaDoors[hKoma].Syurui; //駒は「元・種類」を記憶していませんので、「現・種類」を指定します。
                srcOkiba = Okiba.ShogiBan;
            }
            else
            {
                // 打った駒の場合
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                syurui = uttaSyurui;
                srcSyurui = uttaSyurui;
                switch (kifu.Sengo)
                {
                    case Sengo.Gote:
                        srcOkiba = Okiba.Gote_Komadai;
                        break;
                    case Sengo.Sente:
                        srcOkiba = Okiba.Sente_Komadai;
                        break;
                    default:
                        srcOkiba = Okiba.CTRL_ERROR;
                        break;
                }
            }


            //------------------------------
            // 5
            //------------------------------
            if ("+" == strNari)
            {
                // 成りました
                syurui = Converter01.SyuruiToNariCase(syurui);
            }

            //------------------------------
            // 結果
            //------------------------------
            // 棋譜
            process = new TeProcess(
                kifu.Sengo,
                Okiba.ShogiBan, //符号は将棋盤の升目です。
                srcOkiba,
                suji,
                dan,
                srcSuji,
                srcDan,
                syurui,
                srcSyurui,
                KomaSyurui.CTRL_NOTHING_PROPERTY//取った駒は分からない
            );
        }


    }


}
