﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;

using Xenon.KifuNarabe.L01_Communication;
using Xenon.KifuNarabe.L02_Common;

namespace Xenon.KifuNarabe.L04_Shape
{


    /// <summary>
    /// ************************************************************************************************************************
    /// 描かれる図画です。将棋の駒を描きます。
    /// ************************************************************************************************************************
    /// </summary>
    [Serializable]
    public class Shape_BtnKoma
    {

        #region プロパティー

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 局面の駒リストでの格納位置番号。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int Handle
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 位置とサイズ。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Rectangle Bounds
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 光。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public bool Light
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 動かしたい駒。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public bool Selected
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 動かし終わった駒。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public bool Moved
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 動かしたい駒。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public bool SelectFirstTouch
        {
            get;
            set;
        }

        #endregion


        /// <summary>
        /// ************************************************************************************************************************
        /// コンストラクターです。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="komaPropertyIndex"></param>
        public Shape_BtnKoma(int komaPropertyIndex)
        {
            this.Handle = komaPropertyIndex;
            this.Bounds = new Rectangle(42, 42, 35, 35);
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 駒ボタンの描画はここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="g1"></param>
        /// <param name="kyokumen"></param>
        public void Paint(Graphics g1, Kyokumen kyokumen)
        {
            //----------
            // 背景
            //----------
            if (this.Selected)
            {
                g1.FillRectangle(Brushes.Brown, this.Bounds);
            }
            else if (this.Moved)
            {
                g1.FillRectangle(Brushes.DarkKhaki, this.Bounds);
            }
            else if (this.Light)
            {
            }


            //----------
            // 枠線
            //----------
            {
                Pen pen;
                if (this.Light)
                {
                    pen = Pens.Yellow;
                }
                else
                {
                    pen = Pens.Black;
                }

                if (kyokumen.KomaDoors[this.Handle].Sengo == Sengo.Sente)
                {
                    g1.DrawRectangle(pen, this.Bounds);
                }
                else
                {
                    g1.DrawRectangle(pen, this.Bounds);
                }
            }

            //----------
            // 文字
            //----------
            if (kyokumen.KomaDoors[this.Handle].Sengo == Sengo.Sente)
            {
                string moji = kyokumen.KomaDoors[this.Handle].Text_Label;

                g1.DrawString(moji, new Font(FontFamily.GenericSerif, 20.0f), Brushes.Black, this.Bounds.Location);
            }
            else
            {
                string moji = kyokumen.KomaDoors[this.Handle].Text_Label;

                //----------
                // 使用するフォント
                //----------
                Font fnt = new Font(FontFamily.GenericSerif, 20.0f);

                //----------
                // 文字の大きさにあった白紙（b）
                //----------
                Graphics bG;
                Bitmap bImg;
                {
                    int w;
                    int h;
                    {
                        //----------
                        // 文字の大きさを調べるための白紙（a）
                        //----------
                        Bitmap aImg = new Bitmap(1, 1);

                        //imgのGraphicsオブジェクトを取得
                        Graphics aG = Graphics.FromImage(aImg);

                        //文字列を描画したときの大きさを計算する
                        w = (int)aG.MeasureString(moji, fnt).Width;
                        h = (int)fnt.GetHeight(aG);

                        if (w == 0 || h == 0)
                        {
                            System.Console.WriteLine("moji=["+moji+"]");
                        }

                        if (w < 1)
                        {
                            w = 1;
                        }

                        if (h < 1)
                        {
                            h = 1;
                        }

                        aG.Dispose();
                        aImg.Dispose();
                    }

                    bImg = new Bitmap(w, h);
                }

                // 文字描画
                bG = Graphics.FromImage(bImg);
                bG.DrawString(moji, fnt, Brushes.Black, 0, 0);

                //----------
                // 回転軸座標
                //----------
                float x = (float)this.Bounds.X + (float)this.Bounds.Width / 2;
                float y = (float)this.Bounds.Y + (float)this.Bounds.Height / 2;

                //----------
                // 回転
                //----------

                // 180度で回転するための座標を計算
                //ラジアン単位に変換
                double d = 180 / (180 / Math.PI);
                //新しい座標位置を計算する
                float x1 = x + bImg.Width * (float)Math.Cos(d);
                float y1 = y + bImg.Width * (float)Math.Sin(d);
                float x2 = x - bImg.Height * (float)Math.Sin(d);
                float y2 = y + bImg.Height * (float)Math.Cos(d);
                //PointF配列を作成
                PointF[] destinationPoints = {new PointF(x + (float)this.Bounds.Width / 2, y + (float)this.Bounds.Height / 2),
                                    new PointF(x1 + (float)this.Bounds.Width / 2, y1 + (float)this.Bounds.Height / 2),
                                    new PointF(x2 + (float)this.Bounds.Width / 2, y2 + (float)this.Bounds.Height / 2)};

                //画像を描画
                g1.DrawImage(bImg, destinationPoints);

                //リソースを解放する
                bImg.Dispose();
                bG.Dispose();
                fnt.Dispose();
            }

            ////----------
            //// デバッグ用
            ////----------
            //if (true)
            //{
            //    string moji = this.Masu.ToText();

            //    g1.DrawString(moji, new Font(FontFamily.GenericSerif, 12.0f), Brushes.Red, this.Bounds.Location);
            //}
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// マウスカーソルに重なっているか否か。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public bool HitByMouse(int x, int y)
        {
            return this.Bounds.Contains(x, y);
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// マウスが重なった駒は、光フラグを立てます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public void LightByMouse(int x, int y)
        {
            if (this.HitByMouse(x,y)) // マウスが重なっているなら
            {
                this.Light = true;
            }
            else // マウスが重なっていないなら
            {
                this.Light = false;
            }
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 動かしたい駒
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public void SelectByMouse(int x, int y)
        {
            // 既に選択されている駒には無効
            if (this.Selected)
            {
                goto gt_EndMethod;
            }


            if (this.HitByMouse(x, y)) // マウスが重なっているなら
            {
                this.Selected = true;
                this.SelectFirstTouch = true;
            }
            else
            {
                this.Selected = false;
                this.SelectFirstTouch = false;
            }

        gt_EndMethod:
            ;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 動かしたい駒の解除
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public bool Deselect()
        {
            bool changed = false;

            if (this.SelectFirstTouch)
            {
                // クリックのマウスアップ
                this.SelectFirstTouch = false;
            }
            else
            {
                // 選択解除のマウスアップ
                this.Selected = false;
                changed = true;
            }

            return changed;
        }

    }



}
