﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;

using Xenon.KifuNarabe.L01_Communication;
using Xenon.KifuNarabe.L02_Common;

namespace Xenon.KifuNarabe.L04_Shape
{


    /// <summary>
    /// ************************************************************************************************************************
    /// 描かれる図画です。ボタンとして押せる升目を描きます。
    /// ************************************************************************************************************************
    /// </summary>
    public class Shape_BtnMasu
    {

        #region プロパティー

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 升目の位置。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public TeProcess Masu
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 位置とサイズ。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Rectangle Bounds
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 光。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public bool Light
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 動かしたい駒。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public bool Select
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 動かしたい駒。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public bool SelectFirstTouch
        {
            get;
            set;
        }

        #endregion


        /// <summary>
        /// ************************************************************************************************************************
        /// コンストラクターです。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="okiba"></param>
        /// <param name="suji"></param>
        /// <param name="dan"></param>
        public Shape_BtnMasu(Okiba okiba, int suji, int dan)
        {
            this.Masu = new TeProcess(
                Sengo.Sente,//先後はダミー値。使わないこと。
                okiba,
                suji,
                dan,
                KomaSyurui.CTRL_NOTHING_PROPERTY
                );
            this.Bounds = new Rectangle(42, 42, 35, 35);
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// コンストラクターです。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="okiba"></param>
        /// <param name="suji"></param>
        /// <param name="dan"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Shape_BtnMasu(Okiba okiba, int suji, int dan, int x, int y)
            : this(okiba, suji, dan)
        {
            this.Bounds = new Rectangle(x, y, this.Bounds.Width, this.Bounds.Height);
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 升ボタンの描画はここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="g1"></param>
        public void Paint(Graphics g1)
        {
            //----------
            // 背景
            //----------
            if (this.Select)
            {
                g1.FillRectangle(Brushes.Brown, this.Bounds);
            }
            else if (this.Light)
            {
            }

            //----------
            // 枠線
            //----------
            {
                if (this.Light)
                {
                    g1.DrawRectangle(Pens.Yellow, this.Bounds);
                }
            }
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// マウスカーソルに重なっているか否か。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public bool HitByMouse(int x, int y)
        {
            return this.Bounds.Contains(x, y);
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// マウスが重なった駒は、光フラグを立てます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public void LightByMouse(int x, int y)
        {
            if (this.HitByMouse(x,y)) // マウスが重なっているなら
            {
                this.Light = true;
            }
            else // マウスが重なっていないなら
            {
                this.Light = false;
            }
        }




        /// <summary>
        /// ************************************************************************************************************************
        /// 動かしたい駒の解除
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public bool DeselectByMouse(int x, int y)
        {
            bool changed = false;

            if (this.HitByMouse(x, y)) // マウスが重なっているなら
            {
                if (this.SelectFirstTouch)
                {
                    // クリックのマウスアップ
                    this.SelectFirstTouch = false;
                }
                else
                {
                    // 選択解除のマウスアップ
                    this.Select = false;
                    changed = true;
                }
            }
            else
            {
                // 何もしない
            }

            return changed;
        }
    }


}
