﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;

using Xenon.KifuNarabe.L01_Communication;

namespace Xenon.KifuNarabe.L04_Shape
{

    /// <summary>
    /// ************************************************************************************************************************
    /// 描かれる図画です。駒台を描きます。
    /// ************************************************************************************************************************
    /// </summary>
    public class Shape_PnlKomadai
    {

        #region プロパティー

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 最後の手の駒の筋。「“同”歩」などを調べるのに使う。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int LastSuji
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 最後の手の駒の段。「“同”歩」などを調べるのに使う。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int LastDan
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 駒台の左辺の座標。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int TableX
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 駒台の上辺の座標。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int TableY
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// １升の横幅。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int MasuWidth
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// １升の縦幅。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int MasuHeight
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 駒台が先後どちらのものか。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Sengo Sengo
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// すべての升目１つ１つのリスト。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public List<Shape_BtnMasu> MasuList
        {
            get;
            set;
        }

        #endregion


        /// <summary>
        /// ************************************************************************************************************************
        /// コンストラクターです。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="okiba"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="sengo"></param>
        public Shape_PnlKomadai(Okiba okiba, int x, int y, Sengo sengo)
        {
            this.LastSuji = 4;
            this.LastDan = 10;

            this.TableX = x;
            this.TableY = y;
            this.MasuWidth = 40;
            this.MasuHeight = 40;

            this.Sengo = sengo;

            //----------
            // 升目
            //----------
            this.MasuList = new List<Shape_BtnMasu>();

            for (int suji = 1; suji <= this.LastSuji; suji++)
            {
                for (int dan = 1; dan <= this.LastDan; dan++)
                {
                    this.MasuList.Add(new Shape_BtnMasu(okiba, suji, dan, this.SujiToX(suji), this.DanToY(dan)));
                }
            }
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 筋を指定すると、ｘ座標を返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="suji"></param>
        /// <returns></returns>
        public int SujiToX(int suji)
        {
            return (this.LastSuji - suji) * this.MasuWidth + this.TableX;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 段を指定すると、ｙ座標を返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="dan"></param>
        /// <returns></returns>
        public int DanToY(int dan)
        {
            return (dan-1) * this.MasuHeight + this.TableY;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 駒台の描画は、ここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="g"></param>
        public void Paint(Graphics g)
        {
            //----------
            // 水平線
            //----------
            for (int i = 0; i <= this.LastDan; i++)
            {
                g.DrawLine(Pens.Black,
                                0 * this.MasuWidth + this.TableX,
                                i * this.MasuHeight + this.TableY,
                    this.LastSuji * this.MasuHeight + this.TableX,
                                i * this.MasuHeight + this.TableY
                    );
            }

            //----------
            // 垂直線
            //----------
            for (int i = 0; i <= this.LastSuji; i++)
            {
                g.DrawLine(Pens.Black,
                                i * this.MasuWidth + this.TableX,
                                0 * this.MasuHeight + this.TableY,
                                i * this.MasuHeight + this.TableX,
                    this.LastDan  * this.MasuHeight + this.TableY
                    );
            }

            //----------
            // 升目
            //----------
            foreach (Shape_BtnMasu cell in this.MasuList)
            {
                cell.Paint(g);
            }

        }


    }
}
