﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;

using Xenon.KifuNarabe.L01_Communication;

namespace Xenon.KifuNarabe.L04_Shape
{

    /// <summary>
    /// ************************************************************************************************************************
    /// 描かれる図画です。将棋盤を描きます。
    /// ************************************************************************************************************************
    /// </summary>
    public class Shape_PnlShogiban
    {

        #region プロパティー

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 将棋盤の左辺の座標。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int TableX
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 将棋盤の上辺の座標。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int TableY
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 升の横幅。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int MasuWidth
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 升の縦幅。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public int MasuHeight
        {
            get;
            set;
        }

        #endregion

        /// <summary>
        /// 升目
        /// </summary>
        public List<Shape_BtnMasu> MasuList
        {
            get;
            set;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// コンストラクターです。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Shape_PnlShogiban(int x, int y)
        {
            this.TableX = x;
            this.TableY = y;
            this.MasuWidth = 40;
            this.MasuHeight = 40;

            //----------
            // 升目
            //----------
            this.MasuList = new List<Shape_BtnMasu>();

            for (int suji = 1; suji < 10; suji++)
            {
                for (int dan = 1; dan < 10; dan++)
                {
                    this.MasuList.Add(new Shape_BtnMasu(Okiba.ShogiBan, suji, dan, this.SujiToX(suji), this.DanToY(dan)));
                }
            }

        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 筋を指定すると、ｘ座標を返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="suji"></param>
        /// <returns></returns>
        public int SujiToX(int suji)
        {
            return (9 - suji) * this.MasuWidth + this.TableX;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 段を指定すると、ｙ座標を返します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="dan"></param>
        /// <returns></returns>
        public int DanToY(int dan)
        {
            return (dan-1) * this.MasuHeight + this.TableY;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 将棋盤の描画はここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="g"></param>
        public void Paint(Graphics g)
        {

            //----------
            // 筋の数字
            //----------
            for (int i = 0; i < 9; i++)
            {
                g.DrawString(Converter01.IntToArabiaSuji(i + 1), new Font("ＭＳ ゴシック", 25.0f), Brushes.Black, new Point((8 - i) * this.MasuWidth + this.TableX - 8, -1 * this.MasuHeight + this.TableY));
            }

            //----------
            // 段の数字
            //----------
            for (int i = 0; i < 9; i++)
            {
                g.DrawString(Converter01.IntToKanSuji(i + 1), new Font("ＭＳ ゴシック", 23.0f), Brushes.Black, new Point(9 * this.MasuWidth + this.TableX, i * this.MasuHeight + this.TableY));
                g.DrawString(Converter01.IntToAlphabet(i + 1), new Font("ＭＳ ゴシック", 11.0f), Brushes.Black, new Point(9 * this.MasuWidth + this.TableX, i * this.MasuHeight + this.TableY));
            }


            //----------
            // 水平線
            //----------
            for (int i = 0; i < 10; i++)
            {
                g.DrawLine(Pens.Black,
                    0 * this.MasuWidth + this.TableX,
                    i * this.MasuHeight + this.TableY,
                    9 * this.MasuHeight + this.TableX,
                    i * this.MasuHeight + this.TableY
                    );
            }

            //----------
            // 垂直線
            //----------
            for (int i = 0; i < 10; i++)
            {
                g.DrawLine(Pens.Black,
                    i * this.MasuWidth + this.TableX,
                    0 * this.MasuHeight + this.TableY,
                    i * this.MasuHeight + this.TableX,
                    9 * this.MasuHeight + this.TableY
                    );
            }


            //----------
            // 升目
            //----------
            foreach (Shape_BtnMasu cell in this.MasuList)
            {
                cell.Paint(g);
            }

        }


    }
}
