﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;
using System.Windows.Forms;
using System.Runtime.CompilerServices;
using System.Diagnostics;

using Xenon.KifuNarabe.L01_Communication;
using Xenon.KifuNarabe.L02_Common;

namespace Xenon.KifuNarabe.L04_Shape
{


    /// <summary>
    /// ************************************************************************************************************************
    /// 描かれる図画です。１つの対局で描かれるものは、ここにまとめて入れられています。
    /// ************************************************************************************************************************
    /// </summary>
    public class Shape_PnlTaikyoku
    {


        #region プロパティ類

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// マウスで駒を動かしたときに使います。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 
        /// 棋譜再生中は使いません。
        /// 
        /// </summary>
        public TeProcess MouseProcessOrNull
        {
            get
            {
                return this.mouseProcessOrNull;
            }
        }
        private TeProcess mouseProcessOrNull;


        public void SetMouseProcessOrNull(TeProcess mouse)
        {
            this.mouseProcessOrNull = mouse;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 駒ボタンの配列。局面のKomaDoorsと同じ添え字で一対一対応。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 
        ///     *Doors…名前の由来：ハンドル１つに対応するから。
        /// 
        /// </summary>
        public Shape_BtnKoma[] BtnKomaDoors
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 将棋盤
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_PnlShogiban Shogiban
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 駒置き。[0]先手、[1]後手。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_PnlKomadai[] KomadaiArr
        {
            get;
            set;
        }

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 差し手符号。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        /// <param name="fugo"></param>
        /// <param name="memberName"></param>
        public void SetFugo(
            string fugo,
            [CallerMemberName] string memberName = ""
            )
        {
            this.lblFugo.Text = fugo;

            //System.Console.WriteLine("符号："+fugo+"："+memberName);
        }
        private Shape_LblBox lblFugo;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 先後表示。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        /// <param name="sengo"></param>
        private Shape_LblBox lblSengo;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 成るボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnNaru
        {
            get
            {
                return this.btnNaru;
            }
        }
        private Shape_BtnBox btnNaru;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 向きボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnMuki
        {
            get
            {
                return this.btnMuki;
            }
        }
        private Shape_BtnBox btnMuki;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 初期配置ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnSyokihaichi
        {
            get
            {
                return this.btnSyokihaichi;
            }
        }
        private Shape_BtnBox btnSyokihaichi;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// クリアーボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnClear
        {
            get
            {
                return this.btnClear;
            }
        }
        private Shape_BtnBox btnClear;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// HTMLボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnHtml
        {
            get
            {
                return this.btnHtml;
            }
        }
        private Shape_BtnBox btnHtml;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 再生ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnPlay
        {
            get
            {
                return this.btnPlay;
            }
        }
        private Shape_BtnBox btnPlay;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// コマ送りボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnForward
        {
            get
            {
                return this.btnForward;
            }
        }
        private Shape_BtnBox btnForward;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 戻る
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnBackward
        {
            get
            {
                return this.btnBackward;
            }
        }
        private Shape_BtnBox btnBackward;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 将棋エンジン起動
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnShogiEngineKido
        {
            get
            {
                return this.btnShogiEngineKido;
            }
        }
        private Shape_BtnBox btnShogiEngineKido;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [▲]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Sente
        {
            get
            {
                return this.btnFugo_Sente;
            }
        }
        private Shape_BtnBox btnFugo_Sente;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [△]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Gote
        {
            get
            {
                return this.btnFugo_Gote;
            }
        }
        private Shape_BtnBox btnFugo_Gote;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [１]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_1
        {
            get
            {
                return this.btnFugo_1;
            }
        }
        private Shape_BtnBox btnFugo_1;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [２]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_2
        {
            get
            {
                return this.btnFugo_2;
            }
        }
        private Shape_BtnBox btnFugo_2;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [３]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_3
        {
            get
            {
                return this.btnFugo_3;
            }
        }
        private Shape_BtnBox btnFugo_3;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [４]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_4
        {
            get
            {
                return this.btnFugo_4;
            }
        }
        private Shape_BtnBox btnFugo_4;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [５]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_5
        {
            get
            {
                return this.btnFugo_5;
            }
        }
        private Shape_BtnBox btnFugo_5;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [６]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_6
        {
            get
            {
                return this.btnFugo_6;
            }
        }
        private Shape_BtnBox btnFugo_6;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [７]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_7
        {
            get
            {
                return this.btnFugo_7;
            }
        }
        private Shape_BtnBox btnFugo_7;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [８]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_8
        {
            get
            {
                return this.btnFugo_8;
            }
        }
        private Shape_BtnBox btnFugo_8;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [９]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_9
        {
            get
            {
                return this.btnFugo_9;
            }
        }
        private Shape_BtnBox btnFugo_9;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [同]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Dou
        {
            get
            {
                return this.btnFugo_Dou;
            }
        }
        private Shape_BtnBox btnFugo_Dou;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [歩]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Fu
        {
            get
            {
                return this.btnFugo_Fu;
            }
        }
        private Shape_BtnBox btnFugo_Fu;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [飛]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Hisya
        {
            get
            {
                return this.btnFugo_Hisya;
            }
        }
        private Shape_BtnBox btnFugo_Hisya;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [角]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Kaku
        {
            get
            {
                return this.btnFugo_Kaku;
            }
        }
        private Shape_BtnBox btnFugo_Kaku;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [香]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Kyo
        {
            get
            {
                return this.btnFugo_Kyo;
            }
        }
        private Shape_BtnBox btnFugo_Kyo;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [桂]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Kei
        {
            get
            {
                return this.btnFugo_Kei;
            }
        }
        private Shape_BtnBox btnFugo_Kei;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [銀]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Gin
        {
            get
            {
                return this.btnFugo_Gin;
            }
        }
        private Shape_BtnBox btnFugo_Gin;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [金]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Kin
        {
            get
            {
                return this.btnFugo_Kin;
            }
        }
        private Shape_BtnBox btnFugo_Kin;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [王]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Oh
        {
            get
            {
                return this.btnFugo_Oh;
            }
        }
        private Shape_BtnBox btnFugo_Oh;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [玉]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Gyoku
        {
            get
            {
                return this.btnFugo_Gyoku;
            }
        }
        private Shape_BtnBox btnFugo_Gyoku;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [と]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Tokin
        {
            get
            {
                return this.btnFugo_Tokin;
            }
        }
        private Shape_BtnBox btnFugo_Tokin;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [成香]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Narikyo
        {
            get
            {
                return this.btnFugo_Narikyo;
            }
        }
        private Shape_BtnBox btnFugo_Narikyo;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [成桂]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Narikei
        {
            get
            {
                return this.btnFugo_Narikei;
            }
        }
        private Shape_BtnBox btnFugo_Narikei;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [成銀]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Narigin
        {
            get
            {
                return this.btnFugo_Narigin;
            }
        }
        private Shape_BtnBox btnFugo_Narigin;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [竜]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Ryu
        {
            get
            {
                return this.btnFugo_Ryu;
            }
        }
        private Shape_BtnBox btnFugo_Ryu;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [馬]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Uma
        {
            get
            {
                return this.btnFugo_Uma;
            }
        }
        private Shape_BtnBox btnFugo_Uma;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [寄]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Yoru
        {
            get
            {
                return this.btnFugo_Yoru;
            }
        }
        private Shape_BtnBox btnFugo_Yoru;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [引]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Hiku
        {
            get
            {
                return this.btnFugo_Hiku;
            }
        }
        private Shape_BtnBox btnFugo_Hiku;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [上]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Agaru
        {
            get
            {
                return this.btnFugo_Agaru;
            }
        }
        private Shape_BtnBox btnFugo_Agaru;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [右]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Migi
        {
            get
            {
                return this.btnFugo_Migi;
            }
        }
        private Shape_BtnBox btnFugo_Migi;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [左]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Hidari
        {
            get
            {
                return this.btnFugo_Hidari;
            }
        }
        private Shape_BtnBox btnFugo_Hidari;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [直]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Sugu
        {
            get
            {
                return this.btnFugo_Sugu;
            }
        }
        private Shape_BtnBox btnFugo_Sugu;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [成]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Nari
        {
            get
            {
                return this.btnFugo_Nari;
            }
        }
        private Shape_BtnBox btnFugo_Nari;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [不成]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Funari
        {
            get
            {
                return this.btnFugo_Funari;
            }
        }
        private Shape_BtnBox btnFugo_Funari;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [打]符号ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Da
        {
            get
            {
                return this.btnFugo_Da;
            }
        }
        private Shape_BtnBox btnFugo_Da;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [全消]ボタン     ＜入力欄用＞
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_Zenkesi
        {
            get
            {
                return this.btnFugo_Zenkesi;
            }
        }
        private Shape_BtnBox btnFugo_Zenkesi;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [コンピューターの最善手]ボタン     ＜入力欄用＞
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_ComputernoSaizensyu
        {
            get
            {
                return this.btnFugo_ComputernoSaizensyu;
            }
        }
        private Shape_BtnBox btnFugo_ComputernoSaizensyu;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// [ここから採譜]ボタン
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_BtnBox BtnFugo_KokokaraSaifu
        {
            get
            {
                return this.btnFugo_KokokaraSaifu;
            }
        }
        private Shape_BtnBox btnFugo_KokokaraSaifu;
        //------------------------------------------------------------
        #endregion


        /// <summary>
        /// ************************************************************************************************************************
        /// コンストラクターです。
        /// ************************************************************************************************************************
        /// </summary>
        public Shape_PnlTaikyoku()
        {
            //----------
            // 将ボタン
            //----------
            this.BtnKomaDoors = new Shape_BtnKoma[]{
                // 先手
                new Shape_BtnKoma(0),
                new Shape_BtnKoma(1),
                new Shape_BtnKoma(2),
                new Shape_BtnKoma(3),
                new Shape_BtnKoma(4),
                new Shape_BtnKoma(5),
                new Shape_BtnKoma(6),
                new Shape_BtnKoma(7),
                new Shape_BtnKoma(8),
                new Shape_BtnKoma(9),
                new Shape_BtnKoma(10),
                new Shape_BtnKoma(11),
                new Shape_BtnKoma(12),
                new Shape_BtnKoma(13),
                new Shape_BtnKoma(14),
                new Shape_BtnKoma(15),
                new Shape_BtnKoma(16),
                new Shape_BtnKoma(17),
                new Shape_BtnKoma(18),
                new Shape_BtnKoma(19),

                // 後手
                new Shape_BtnKoma(20),
                new Shape_BtnKoma(21),
                new Shape_BtnKoma(22),
                new Shape_BtnKoma(23),
                new Shape_BtnKoma(24),
                new Shape_BtnKoma(25),
                new Shape_BtnKoma(26),
                new Shape_BtnKoma(27),
                new Shape_BtnKoma(28),
                new Shape_BtnKoma(29),
                new Shape_BtnKoma(30),
                new Shape_BtnKoma(31),
                new Shape_BtnKoma(32),
                new Shape_BtnKoma(33),
                new Shape_BtnKoma(34),
                new Shape_BtnKoma(35),
                new Shape_BtnKoma(36),
                new Shape_BtnKoma(37),
                new Shape_BtnKoma(38),
                new Shape_BtnKoma(39)
            };

            //----------
            // 将棋盤
            //----------
            this.Shogiban = new Shape_PnlShogiban(200, 220);

            //----------
            // 駒置き
            //----------
            this.KomadaiArr = new Shape_PnlKomadai[2];
            this.KomadaiArr[0] = new Shape_PnlKomadai(Okiba.Sente_Komadai, 610, 220, Sengo.Sente);
            this.KomadaiArr[1] = new Shape_PnlKomadai(Okiba.Gote_Komadai, 10, 170, Sengo.Gote);

            //----------
            // 符号表示
            //----------
            this.lblFugo = new Shape_LblBox("符号", 480, 145);

            //----------
            // 先後表示
            //----------
            this.lblSengo = new Shape_LblBox("－－", 350, 145);

            //----------
            // 成るボタン
            //----------
            this.btnNaru = new Shape_BtnBox("成る", 250, 590);

            //----------
            // 向きボタン
            //----------
            this.btnMuki = new Shape_BtnBox("向き", 330, 590);

            //----------
            // 初期配置ボタン
            //----------
            this.btnSyokihaichi = new Shape_BtnBox("初期配置", 450, 590);

            //----------
            // クリアーボタン
            //----------
            this.btnClear = new Shape_BtnBox("クリアー", 450, 630);

            //----------
            // HTMLボタン
            //----------
            this.btnHtml = new Shape_BtnBox("HTML", 570, 630);

            //----------
            // 再生ボタン
            //----------
            this.btnPlay = new Shape_BtnBox("再生", 710, 0);

            //----------
            // コマ送りボタン
            //----------
            this.btnForward = new Shape_BtnBox("コマ送り", 710, 40);

            //----------
            // 戻るボタン
            //----------
            this.btnBackward = new Shape_BtnBox("戻る", 710, 80);

            //----------
            // 将棋エンジン起動ボタン
            //----------
            this.btnShogiEngineKido = new Shape_BtnBox("将棋エンジン起動", 600, 120);

            //----------
            // [▲]～[打]符号ボタン
            //----------
            int ox = 30;
            int oy = 40;
            int w = 23;
            int h = 23;
            float fontSize = 13.0f;
            this.btnFugo_Sente = new Shape_BtnBox("▲", ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_Gote = new Shape_BtnBox("△", ox, 1 * h + oy, w, h, fontSize);
            ox += w;

            this.btnFugo_1 = new Shape_BtnBox("１", 0 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_2 = new Shape_BtnBox("２", 1 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_3 = new Shape_BtnBox("３", 2 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_4 = new Shape_BtnBox("４", 0 * w + ox, 1 * h + oy, w, h, fontSize);
            this.btnFugo_5 = new Shape_BtnBox("５", 1 * w + ox, 1 * h + oy, w, h, fontSize);
            this.btnFugo_6 = new Shape_BtnBox("６", 2 * w + ox, 1 * h + oy, w, h, fontSize);
            this.btnFugo_7 = new Shape_BtnBox("７", 0 * w + ox, 2 * h + oy, w, h, fontSize);
            this.btnFugo_8 = new Shape_BtnBox("８", 1 * w + ox, 2 * h + oy, w, h, fontSize);
            this.btnFugo_9 = new Shape_BtnBox("９", 2 * w + ox, 2 * h + oy, w, h, fontSize);
            this.btnFugo_Dou = new Shape_BtnBox("同", 3 * w + ox, 0 * h + oy, w, h, fontSize);
            ox += (4 + 1) * w;

            this.btnFugo_Fu = new Shape_BtnBox("歩", 0 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_Hisya = new Shape_BtnBox("飛", 1 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_Kaku = new Shape_BtnBox("角", 2 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_Kyo = new Shape_BtnBox("香", 3 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_Kei = new Shape_BtnBox("桂", 4 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_Gin = new Shape_BtnBox("銀", 5 * w + ox, 0 * h + oy, w, h, fontSize);

            this.btnFugo_Kin = new Shape_BtnBox("金", 0 * w + ox, 1 * h + oy, w, h, fontSize);
            this.btnFugo_Oh = new Shape_BtnBox("王", 1 * w + ox, 1 * h + oy, w, h, fontSize);
            this.btnFugo_Gyoku = new Shape_BtnBox("玉", 2 * w + ox, 1 * h + oy, w, h, fontSize);
            this.btnFugo_Tokin = new Shape_BtnBox("と", 3 * w + ox, 1 * h + oy, w, h, fontSize);
            this.btnFugo_Narikyo = new Shape_BtnBox("成香", 4 * w + ox, 1 * h + oy, 2 * w, h, fontSize);

            this.btnFugo_Narikei = new Shape_BtnBox("成桂", 0 * w + ox, 2 * h + oy, 2 * w, h, fontSize);
            this.btnFugo_Narigin = new Shape_BtnBox("成銀", 2 * w + ox, 2 * h + oy, 2 * w, h, fontSize);
            this.btnFugo_Ryu = new Shape_BtnBox("竜", 4 * w + ox, 2 * h + oy, w, h, fontSize);
            this.btnFugo_Uma = new Shape_BtnBox("馬", 5 * w + ox, 2 * h + oy, w, h, fontSize);
            ox += (6 + 1) * w;

            this.btnFugo_Yoru = new Shape_BtnBox("寄", 0 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_Hiku = new Shape_BtnBox("引", 1 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_Agaru = new Shape_BtnBox("上", 2 * w + ox, 0 * h + oy, w, h, fontSize);

            this.btnFugo_Migi = new Shape_BtnBox("右", 0 * w + ox, 1 * h + oy, w, h, fontSize);
            this.btnFugo_Hidari = new Shape_BtnBox("左", 1 * w + ox, 1 * h + oy, w, h, fontSize);
            this.btnFugo_Sugu = new Shape_BtnBox("直", 2 * w + ox, 1 * h + oy, w, h, fontSize);

            this.btnFugo_Nari = new Shape_BtnBox("成", 0 * w + ox, 2 * h + oy, w, h, fontSize);
            this.btnFugo_Funari = new Shape_BtnBox("不成", 1 * w + ox, 2 * h + oy, 2 * w, h, fontSize);
            this.btnFugo_Da = new Shape_BtnBox("打", 3 * w + ox, 2 * h + oy, w, h, fontSize);
            this.btnFugo_Zenkesi = new Shape_BtnBox("全消", 5 * w + ox, 0 * h + oy, w, h, fontSize);
            this.btnFugo_ComputernoSaizensyu = new Shape_BtnBox("コンピューターの最善手（作りかけ）", 5 * w + ox, 2 * h + oy, w, h, fontSize);

            this.btnFugo_KokokaraSaifu = new Shape_BtnBox("ここから採譜", 9 * w + ox, 0 * h + oy, w, h, fontSize);

        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 対局の描画の一式は、ここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Paint(object sender, PaintEventArgs e, Kifu kifu)
        {
            //----------
            // 将棋盤
            //----------
            this.Shogiban.Paint(e.Graphics);

            //----------
            // 駒置き
            //----------
            foreach (Shape_PnlKomadai k in this.KomadaiArr)
            {
                k.Paint(e.Graphics);
            }

            //----------
            // 駒
            //----------
            foreach (Shape_BtnKoma koma in this.BtnKomaDoors)
            {
                koma.Paint(e.Graphics, kifu.Kyokumen);
            }

            //----------
            // 符号表示
            //----------
            this.lblFugo.Paint(e.Graphics);

            //----------
            // 先後表示
            //----------
            this.lblSengo.Text = Converter01.SengoToKanji(kifu.Sengo);
            this.lblSengo.Paint(e.Graphics);

            //----------
            // 成るボタン
            //----------
            this.BtnNaru.Paint(e.Graphics);

            //----------
            // 向きボタン
            //----------
            this.BtnMuki.Paint(e.Graphics);

            //----------
            // 初期配置ボタン
            //----------
            this.BtnSyokihaichi.Paint(e.Graphics);

            //----------
            // クリアーボタン
            //----------
            this.BtnClear.Paint(e.Graphics);

            //----------
            // HTMLボタン
            //----------
            this.BtnHtml.Paint(e.Graphics);

            //----------
            // 再生ボタン
            //----------
            this.BtnPlay.Paint(e.Graphics);

            //----------
            // コマ送りボタン
            //----------
            this.BtnForward.Paint(e.Graphics);

            //----------
            // 戻るボタン
            //----------
            this.BtnBackward.Paint(e.Graphics);

            //----------
            // 将棋エンジン起動ボタン
            //----------
            this.BtnShogiEngineKido.Paint(e.Graphics);

            //----------
            // [▲]～[打]符号ボタン
            //----------
            this.btnFugo_Sente.Paint(e.Graphics);
            this.btnFugo_Gote.Paint(e.Graphics);
            this.btnFugo_1.Paint(e.Graphics);
            this.btnFugo_2.Paint(e.Graphics);
            this.btnFugo_3.Paint(e.Graphics);
            this.btnFugo_4.Paint(e.Graphics);
            this.btnFugo_5.Paint(e.Graphics);
            this.btnFugo_6.Paint(e.Graphics);
            this.btnFugo_7.Paint(e.Graphics);
            this.btnFugo_8.Paint(e.Graphics);
            this.btnFugo_9.Paint(e.Graphics);
            this.btnFugo_Dou.Paint(e.Graphics);
            this.btnFugo_Fu.Paint(e.Graphics);
            this.btnFugo_Hisya.Paint(e.Graphics);
            this.btnFugo_Kaku.Paint(e.Graphics);
            this.btnFugo_Kyo.Paint(e.Graphics);
            this.btnFugo_Kei.Paint(e.Graphics);
            this.btnFugo_Gin.Paint(e.Graphics);
            this.btnFugo_Kin.Paint(e.Graphics);
            this.btnFugo_Oh.Paint(e.Graphics);
            this.btnFugo_Gyoku.Paint(e.Graphics);
            this.btnFugo_Tokin.Paint(e.Graphics);
            this.btnFugo_Narikyo.Paint(e.Graphics);
            this.btnFugo_Narikei.Paint(e.Graphics);
            this.btnFugo_Narigin.Paint(e.Graphics);
            this.btnFugo_Ryu.Paint(e.Graphics);
            this.btnFugo_Uma.Paint(e.Graphics);
            this.btnFugo_Yoru.Paint(e.Graphics);
            this.btnFugo_Hiku.Paint(e.Graphics);
            this.btnFugo_Agaru.Paint(e.Graphics);
            this.btnFugo_Migi.Paint(e.Graphics);
            this.btnFugo_Hidari.Paint(e.Graphics);
            this.btnFugo_Sugu.Paint(e.Graphics);
            this.btnFugo_Nari.Paint(e.Graphics);
            this.btnFugo_Funari.Paint(e.Graphics);
            this.btnFugo_Da.Paint(e.Graphics);
            this.btnFugo_Zenkesi.Paint(e.Graphics);
            this.btnFugo_ComputernoSaizensyu.Paint(e.Graphics);
            this.btnFugo_KokokaraSaifu.Paint(e.Graphics);
        }


        /// <summary>
        /// 移動直後の駒を取得。
        /// </summary>
        /// <returns>移動直後の駒。なければヌル</returns>
        public Shape_BtnKoma Btn_MovedKoma()
        {
            Shape_BtnKoma btn = null;

            foreach (Shape_BtnKoma btnKoma in this.BtnKomaDoors)
            {
                if (btnKoma.Moved)
                {
                    btn = btnKoma;
                }
            }

            return btn;
        }

        /// <summary>
        /// 各駒の、移動済フラグを解除
        /// </summary>
        public void ClearAllMoverdFlag()
        {
            foreach (Shape_BtnKoma btnKoma in this.BtnKomaDoors)
            {
                btnKoma.Moved = false;
            }
        }


        /// <summary>
        /// 各駒の、移動済フラグを解除
        /// </summary>
        public void ClearAllSelectedFlag()
        {
            foreach (Shape_BtnKoma btnKoma in this.BtnKomaDoors)
            {
                btnKoma.Selected = false;
                btnKoma.SelectFirstTouch = false;
            }
        }

        /// <summary>
        /// 選択されている駒を取得。
        /// </summary>
        /// <returns>選択されている駒。なければヌル</returns>
        public Shape_BtnKoma Btn_Selected()
        {
            Shape_BtnKoma found = null;

            foreach (Shape_BtnKoma btnKoma in this.BtnKomaDoors)
            {
                if (btnKoma.Selected)
                {
                    found = btnKoma;
                }
            }

            return found;
        }

    }


}
