﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Diagnostics;
using System.IO;
using Xenon.KifuNarabe.L00_DammyConsole;
using Xenon.KifuNarabe.L01_Communication;
using Xenon.KifuNarabe.L02_Common;
using Xenon.KifuNarabe.L06_Ui;

namespace Xenon.KifuNarabe.L05_Server
{


    /// <summary>
    /// ************************************************************************************************************************
    /// １つの将棋エンジンと通信します。１対１の関係になります。
    /// ************************************************************************************************************************
    /// </summary>
    public class ShogiEngineService
    {


        #region プロパティ類

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// これが、将棋エンジン（プロセス）です。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public static Process shogiEngineProcess;

        #endregion

        /// <summary>
        /// 将棋エンジンが起動しているか否かです。
        /// </summary>
        /// <returns></returns>
        public static bool IsLiveShogiEngine()
        {
            return null != ShogiEngineService.shogiEngineProcess && !ShogiEngineService.shogiEngineProcess.HasExited;
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 将棋エンジンを起動します。
        /// ************************************************************************************************************************
        /// </summary>
        public static void StartShogiEngine()
        {
            try
            {
                if (ShogiEngineService.IsLiveShogiEngine())//null!=ShogiEngineService.shogiEngineProcess && !ShogiEngineService.shogiEngineProcess.HasExited
                {
                    System.Console.WriteLine("将棋エンジンサービスは終了していません。");
                    goto gt_EndMethod;
                }

                ProcessStartInfo startInfo = new ProcessStartInfo();

                startInfo.FileName = @"shogiEngine.exe"; // 実行するファイル名決め打ち
                //startInfo.CreateNoWindow = true; // コンソール・ウィンドウを開かない
                startInfo.UseShellExecute = false; // シェル機能を使用しない
                startInfo.RedirectStandardInput = true;//標準入力をリダイレクト
                startInfo.RedirectStandardOutput = true; // 標準出力をリダイレクト

                ShogiEngineService.shogiEngineProcess = Process.Start(startInfo); // アプリの実行開始

                //  OutputDataReceivedイベントハンドラを追加
                ShogiEngineService.shogiEngineProcess.OutputDataReceived += ShogiEngineService.ReceivedData_FromShogiEngine_Async;
                ShogiEngineService.shogiEngineProcess.Exited += ShogiEngineService.ShogiEngineService_Exited;

                // 非同期受信スタート☆！
                ShogiEngineService.shogiEngineProcess.BeginOutputReadLine();

                // 「usi」
                ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("usi");


                // エンジン設定
                // 「setoption」
                //Server.shogiEngineProcess.StandardInput.WriteLine("setoption");


                //
                // サーバー・スレッドはここで終了しますが、
                // 代わりに、Server.shogiEngineProcess プロセスが動き続け、
                // ReceivedData_FromShogiEngine メソッドが非同期に呼び出され続けます。
                //

                //this.thread = new Thread(new ThreadStart(Server.Main2));
                //this.thread.IsBackground = true;
                //this.thread.Start();

            }
            catch (Exception ex)
            {
                System.Console.WriteLine(ex.GetType().Name + "：" + ex.Message);
            }

        gt_EndMethod:
            ;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// このサービスを終了したときにする挙動を、ここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public static void ShogiEngineService_Exited(object sender, System.EventArgs e)
        {
            ShogiEngineService.Shutdown();
        }

        public static void Shutdown()
        {
            //------------------------------------------------------------
            // 将棋エンジンに、終了するように促します。
            //------------------------------------------------------------
            if (ShogiEngineService.IsLiveShogiEngine())
            {
                ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("quit");
            }
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 手番が替わったときの挙動を、ここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        public static void ChangeTurn(Kifu kifu)
        {

            if (!ShogiEngineService.IsLiveShogiEngine())
            {
                goto gt_EndMethod;
            }

            switch (kifu.Sengo)
            {
                case Sengo.Gote:
                    // 仮に、コンピューターが後手番とします。

                    //------------------------------------------------------------
                    // とりあえず、コンピューターが後手ということにしておきます。
                    //------------------------------------------------------------
                    ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("position startpos moves 7g7f");//FIXME:嘘

                    ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("go");

                    break;
                default:
                    break;
            }

        gt_EndMethod:
            ;
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 将棋エンジンから、データを非同期受信(*1)します。
        /// ************************************************************************************************************************
        /// 
        ///         *1…こっちの都合に合わせず、データが飛んできます。
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public static void ReceivedData_FromShogiEngine_Async(object sender, System.Diagnostics.DataReceivedEventArgs e)
        {
            string line = e.Data;
            //Console.WriteLine("Ｓ：非同期："+line);

            if (null == line)
            {
                // 無視
            }
            else
            {
                System.Console.WriteLine("Ｓ：受信=[" + line + "] len=" + line.Length);

                if (line.StartsWith("option"))
                {

                }
                else if ("usiok" == line)
                {
                    ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("isready");
                }
                else if ("readyok" == line)
                {

                    //------------------------------------------------------------
                    // 対局開始！
                    //------------------------------------------------------------
                    ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("usinewgame");

                    ////------------------------------------------------------------
                    //// とりあえず、コンピューターが後手ということにしておきます。
                    ////------------------------------------------------------------
                    //ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("position startpos moves 7g7f");//FIXME:嘘

                    //ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("go");
                }
                else if (line.StartsWith("info"))
                {
                }
                else if (line.StartsWith("bestmove resign"))
                {
                    // 将棋エンジンが、投了されました。
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                    //------------------------------------------------------------
                    // あなたの負けです☆
                    //------------------------------------------------------------
                    ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("gameover lose");

                    //------------------------------------------------------------
                    // 将棋エンジンを終了してください☆
                    //------------------------------------------------------------
                    ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("quit");
                }
                else if (line.StartsWith("bestmove"))
                {
                    // 将棋エンジンが、手を指されました。
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                    Ui_PnlMain.input99 += line.Substring("bestmove".Length+"".Length);
                    System.Console.WriteLine("将棋エンジンが、手を指されました。Ui_PnlMain.input99=[" + Ui_PnlMain.input99 + "]");

                    //スレッド違反
                    //DefaultShogiEngine.ShogiEngine.WriteLine1(line.Substring("bestmove".Length));

                    //ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("position startpos moves 7g7f");//FIXME:嘘

                    //ShogiEngineService.shogiEngineProcess.StandardInput.WriteLine("go");
                }
                else
                {
                }
            }
        }
    }


}
