﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Windows.Forms;
using Xenon.KifuNarabe.L00_DammyConsole;
using Xenon.KifuNarabe.L01_Communication;
using Xenon.KifuNarabe.L02_Common;
using Xenon.KifuNarabe.L03_TuginoItte;
using Xenon.KifuNarabe.L04_Shape;

namespace Xenon.KifuNarabe.L06_Ui
{

    /// <summary>
    /// ************************************************************************************************************************
    /// UI(*1)に関する操作です。
    /// ************************************************************************************************************************
    /// 
    ///         *1…目に触れ、入力のために触る部分です。ユーザーインターフェース。ボタンやラベルなど。
    /// 
    /// </summary>
    public abstract class Ui_02Menu
    {

        /// <summary>
        /// ************************************************************************************************************************
        /// [戻る]ボタン
        /// ************************************************************************************************************************
        /// </summary>
        public static bool Modoru( Shape_PnlTaikyoku brush_PnlTaikyoku, Kifu kifu, out string backedInputText)
        {
            bool successful = false;
            backedInputText = DammyConsole.DefaultDammyConsole.ReadLine1().Trim();

            //------------------------------
            // 棋譜から１手削ります
            //------------------------------
            TeProcess backedProcess;
            kifu.PopLast(out backedProcess);

            if (null == backedProcess)
            {
                goto gt_EndMethod;
            }


            //------------------------------
            // 入力欄
            //------------------------------
            Fugo lastFugo;
            lastFugo = TuginoItte_JapanFugo.GetText_FromData(backedProcess, kifu.Kyokumen);
            TeProcess doubleBackedProcess = kifu.Last;
            string lastFutoStr = lastFugo.ToText(true, backedProcess, doubleBackedProcess);
            backedInputText = lastFutoStr + " " + backedInputText;
            


            //------------------------------
            // 前の手に戻します
            //------------------------------
            Ui_03Action.Kifusasi(backedProcess, brush_PnlTaikyoku, kifu, true);



            //------------------------------
            // 符号表示
            //------------------------------
            brush_PnlTaikyoku.SetFugo(lastFutoStr);

            //------------------------------
            // チェンジターン
            //------------------------------
            kifu.ChangeTurn(backedProcess.Suji, backedProcess.Dan);

            successful = true;


        gt_EndMethod:
            return successful;
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// [コマ送り]ボタン
        /// ************************************************************************************************************************
        /// </summary>
        public static bool Komaokuri(ref RequestForMain requestForMain, Shape_PnlTaikyoku brush_PnlTaikyoku, Kifu kifu)
        {

            //
            // 次のような状況を想定しています。
            //
            //      「次の一手は、もう将棋エンジンに溜められていて、ReadLine() で取り出せる」
            //

            return Ui_02Menu.ReadLine_TuginoItteSusumu(ref requestForMain, brush_PnlTaikyoku, kifu);
        }



        /// <summary>
        /// ************************************************************************************************************************
        /// １手進む
        /// ************************************************************************************************************************
        /// 
        /// ＜棋譜読込用＞＜マウス入力非対応＞
        /// </summary>
        public static bool ReadLine_TuginoItteSusumu(
            ref RequestForMain requestForMain,
            Shape_PnlTaikyoku brush_PnlTaikyoku,
            Kifu kifu
            )
        {
            bool successful = false;

            TeProcess process;
            string restText;

            string inputLine = DammyConsole.DefaultDammyConsole.ReadLine1().Trim();

            if (inputLine.StartsWith("position"))
            {
                // SFEN形式の「position」コマンドが、入力欄に入っていました。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                //------------------------------------------------------------
                // まずこのブロックで「position ～ moves 」まで(*1)を処理します。
                //------------------------------------------------------------
                //
                //          *1…初期配置を作るということです。
                // 

                inputLine = inputLine.Substring("position".Length);
                inputLine = inputLine.Trim();
                successful = true;

                bool nextMoves = false;

                if (inputLine.StartsWith("startpos"))
                {
                    // 平手の初期配置です。
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                    inputLine = inputLine.Substring("startpos".Length);
                    inputLine = inputLine.Trim();

                    Ui_03Action.Syokihaichi_Hirate( kifu);
                    // 再描画
                    foreach (Shape_BtnKoma btnKoma in brush_PnlTaikyoku.BtnKomaDoors)
                    {
                        Ui_03Action.Refresh_KomaLocation(btnKoma.Handle, brush_PnlTaikyoku, kifu.Kyokumen);
                    }

                    requestForMain.RequestClearTxtOutput = true;
                    requestForMain.RequestRefresh = true;

                    nextMoves = true;
                }
                else
                {
                    //　「lnsgkgsnl/1r5b1/ppppppppp/9/9/6P2/PPPPPP1PP/1B5R1/LNSGKGSNL w - 1」といった文字の読込み

                    SfenStartpos sfenStartpos;

                    if (!TuginoItte_Sfen.GetDataStartpos_FromText(inputLine, out restText, out sfenStartpos))
                    {
                        // 解析に失敗しました。
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                    }
                    else
                    {
                        Ui_03Action.Syokihaichi_ByStartpos(sfenStartpos, kifu);

                        // 駒袋に表示されている駒を、駒台に表示させます。
                        Ui_03Action.Refresh_KomabukuroToKomadai(sfenStartpos, brush_PnlTaikyoku, kifu);

                        //------------------------------
                        // 持ち駒が動いているので、全駒、再描画
                        //------------------------------
                        for (int hKoma = 0; hKoma < kifu.Kyokumen.KomaDoors.Length; hKoma++)
                        {
                            Ui_03Action.Refresh_KomaLocation(hKoma, brush_PnlTaikyoku, kifu.Kyokumen);
                        }

                        inputLine = restText;
                        requestForMain.RequestClearTxtOutput = true;
                        requestForMain.RequestRefresh = true;

                        nextMoves = true;
                    }
                }

                if (nextMoves)
                {
                    if (inputLine.StartsWith("moves"))
                    {
                        // 棋譜が始まります。
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        inputLine = inputLine.Substring("moves".Length);
                        inputLine = inputLine.Trim();
                    }
                }

            }
            // ここまでで、「position ～ moves 」といった記述が入力されていたとすれば、入力欄から削除済みです。


            //「▲６六歩」形式と想定して、１手だけ読込み
            if (!TuginoItte_JapanFugo.GetData_FromText(
                inputLine, out restText, out process, kifu))
            {
                //「6g6f」形式と想定して、１手だけ読込み
                if (!TuginoItte_Sfen.GetData_FromText(
                    inputLine, out restText, out process, kifu))
                {

                }
            }
            inputLine = restText;


            if (null != process)
            {
                Application.DoEvents(); // 時間のかかる処理の間にはこれを挟みます。

                //------------------------------------------------------------
                // 1手ずつ
                //------------------------------------------------------------
                TeProcess previousProcess = kifu.Last; //符号の追加が行われる前に退避

                //------------------------------
                // 符号の追加
                //------------------------------
                kifu.Add(process);
                Fugo fugo = TuginoItte_JapanFugo.GetText_FromData(process, kifu.Kyokumen);

                //------------------------------
                // ★棋譜読込専用  駒移動
                //------------------------------
                Ui_03Action.Kifusasi(process, brush_PnlTaikyoku, kifu, false);

                //------------------------------
                // 符号表示
                //------------------------------
                brush_PnlTaikyoku.SetFugo(fugo.ToText(true, process, previousProcess));

                requestForMain.RequestOutputKifu = true;   // 棋譜出力要求

                //------------------------------
                // チェンジターン
                //------------------------------
                TeProcess last = kifu.Last;
                kifu.ChangeTurn(last.Suji, last.Dan);
                requestForMain.RequestRefresh = true;

                successful = true;
            }

            requestForMain.RequestInputTextString = inputLine;

            return successful;
        }




        /// <summary>
        /// ************************************************************************************************************************
        /// 将棋盤の上の駒を、全て駒台に移動します。（クリアー）
        /// ************************************************************************************************************************
        /// </summary>
        public static void ClearKifu(Kifu kifu)
        {
            Kyokumen ky = kifu.Kyokumen;

            int i = 0;
            kifu.Init();


            // 先手
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 1, 1, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 2, 1, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 3, 1, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 4, 1, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 1, 2, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 2, 2, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 3, 2, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 4, 2, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 1, 3, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 2, 3, ky.KomaDoors[i].Syurui); //飛
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 3, 3, ky.KomaDoors[i].Syurui); //角
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 4, 3, ky.KomaDoors[i].Syurui); //香
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 1, 4, ky.KomaDoors[i].Syurui); //桂
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 2, 4, ky.KomaDoors[i].Syurui); //銀
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 3, 4, ky.KomaDoors[i].Syurui); //金
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 4, 4, ky.KomaDoors[i].Syurui); //王
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 1, 5, ky.KomaDoors[i].Syurui); //金
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 2, 5, ky.KomaDoors[i].Syurui); //銀
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 3, 5, ky.KomaDoors[i].Syurui); //桂
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Sente, Okiba.Sente_Komadai, 4, 5, ky.KomaDoors[i].Syurui); //香
            i++;


            // 後手
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 1, 1, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 2, 1, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 3, 1, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 4, 1, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 1, 2, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 2, 2, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 3, 2, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 4, 2, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 1, 3, ky.KomaDoors[i].Syurui); //歩
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 2, 3, ky.KomaDoors[i].Syurui); //飛
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 3, 3, ky.KomaDoors[i].Syurui); //角
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 4, 3, ky.KomaDoors[i].Syurui); //香
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 1, 4, ky.KomaDoors[i].Syurui); //桂
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 2, 4, ky.KomaDoors[i].Syurui); //銀
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 3, 4, ky.KomaDoors[i].Syurui); //金
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 4, 4, ky.KomaDoors[i].Syurui); //玉
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 1, 5, ky.KomaDoors[i].Syurui); //金
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 2, 5, ky.KomaDoors[i].Syurui); //銀
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 3, 5, ky.KomaDoors[i].Syurui); //桂
            i++;
            ky.KomaDoors[i] = new TeProcess(Sengo.Gote, Okiba.Gote_Komadai, 4, 5, ky.KomaDoors[i].Syurui); //香
            i++;


            // 符号の記憶を消去します。
            kifu.Clear();
            kifu.SetStartpos("9/9/9/9/9/9/9/9/9 b K1R1B1G2S2N2L2P9 k1r1b1g2s2n2l2p9 1");
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// HTML出力。（これは作者のホームページ用に書かれています）
        /// ************************************************************************************************************************
        /// </summary>
        public static string CreateHtml(Kyokumen kyokumen)
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendLine("<div style=\"position:relative; left:0px; top:0px; border:solid 1px black; width:250px; height:180px;\">");

            // 後手の持ち駒
            sb.AppendLine("    <div style=\"position:absolute; left:0px; top:2px; width:30px;\">");
            sb.AppendLine("        △後手");
            sb.AppendLine("        <div style=\"margin-top:10px; width:30px;\">");
            sb.Append("            ");
            foreach (TeProcess koma in kyokumen.KomaDoors)
            {
                if (koma.MatchOkiba(Okiba.Gote_Komadai))
                {
                    sb.Append(Converter01.SyuruiToFugo(koma.Syurui));
                }
            }
            sb.AppendLine("        </div>");
            sb.AppendLine("    </div>");

            // 将棋盤
            sb.AppendLine("    <div style=\"position:absolute; left:30px; top:2px; width:182px;\">");
            sb.AppendLine("        <table>");
            for (int dan = 1; dan <= 9; dan++)
            {
                sb.Append("        <tr>");
                for (int suji = 9; 1 <= suji; suji--)
                {
                    bool isSpace = true;

                    foreach (TeProcess koma in kyokumen.KomaDoors)
                    {
                        if (
                            koma.MatchOkiba(Okiba.ShogiBan) //盤上
                            && koma.MatchSuji(suji)
                            && koma.MatchDan(dan)
                        )
                        {
                            if (Sengo.Gote == koma.Sengo)
                            {
                                sb.Append("<td><span class=\"koma2x\">");
                                sb.Append(Converter01.SyuruiToFugo(koma.Syurui));
                                sb.Append("</span></td>");
                                isSpace = false;
                            }
                            else
                            {
                                sb.Append("<td><span class=\"koma1x\">");
                                sb.Append(Converter01.SyuruiToFugo(koma.Syurui));
                                sb.Append("</span></td>");
                                isSpace = false;
                            }
                        }
                    }

                    if (isSpace)
                    {
                        sb.Append("<td>　</td>");
                    }
                }

                sb.AppendLine("</tr>");
            }
            sb.AppendLine("        </table>");
            sb.AppendLine("    </div>");

            // 先手の持ち駒
            sb.AppendLine("    <div style=\"position:absolute; left:215px; top:2px; width:30px;\">");
            sb.AppendLine("        ▲先手");
            sb.AppendLine("        <div style=\"margin-top:10px; width:30px;\">");
            sb.Append("            ");
            foreach (TeProcess koma in kyokumen.KomaDoors)
            {
                if (koma.MatchOkiba(Okiba.Sente_Komadai))
                {
                    sb.Append(Converter01.SyuruiToFugo(koma.Syurui));
                }
            }
            sb.AppendLine("        </div>");
            sb.AppendLine("    </div>");

            //
            sb.AppendLine("</div>");

            return sb.ToString();
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 棋譜データを元に、符号リスト１(*1)を出力します。
        /// ************************************************************************************************************************
        /// 
        ///     *1…「▲２六歩△８四歩▲７六歩」といった書き方。
        /// 
        /// </summary>
        /// <param name="fugoList"></param>
        public static string ToKifu1Text(Kifu kifu)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("position ");
            sb.Append(kifu.Startpos);
            sb.Append(" moves ");

            // 採譜用に、新しい対局を用意します。
            Kifu saifuKifu = new Kifu();
            Shape_PnlTaikyoku saifuBrush_PnlTaikyoku = new Shape_PnlTaikyoku();

            saifuKifu.SetStartpos(kifu.Startpos);

            // 初期配置
            Ui_03Action.Syokihaichi_Hirate(saifuKifu);
            //// 再描画
            //foreach (Brush_BtnKoma btnKoma in brush_PnlTaikyoku.BtnKomaDoors)
            //{
            //    Ui_04SubAction.RefreshKomaLocation(btnKoma, brush_PnlTaikyoku, kifu.Kyokumen);
            //}


            foreach (TeProcess process in kifu.ProcessList)
            {
                TeProcess previousMasu = saifuKifu.Last; //符号の追加が行われる前に退避

                Fugo fugo;

                //------------------------------
                // 符号の追加
                //------------------------------
                saifuKifu.Add(process);
                fugo = TuginoItte_JapanFugo.GetText_FromData(process, saifuKifu.Kyokumen);

                sb.Append(fugo.ToText(true, process, previousMasu));

                //// TODO:デバッグ用
                //switch (process.TottaKoma)
                //{
                //    case KomaSyurui.UNKNOWN:
                //    case KomaSyurui.TOTTA_KOMA_NASI:
                //        break;
                //    default:
                //        sb.Append("（");
                //        sb.Append(Converter.SyuruiToFugo(process.TottaKoma));
                //        sb.Append("取り）");
                //        break;
                //}
            }

            //System.Console.WriteLine("size(" + fugoList.Count+")"+sb.ToString());

            return sb.ToString();
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 棋譜データを元に、符号リスト２(*1)を出力します。
        /// ************************************************************************************************************************
        /// 
        ///     *1…「position startpos moves 7g7f 3c3d 2g2f」といった書き方。
        /// 
        /// </summary>
        /// <param name="fugoList"></param>
        public static string ToKifu2Text(Kifu kifu)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("position ");
            sb.Append(kifu.Startpos);
            sb.Append(" moves ");

            foreach (TeProcess process in kifu.ProcessList)
            {
                sb.Append(process.ToSfenText());

                //// TODO:デバッグ用
                //switch (process.TottaKoma)
                //{
                //    case KomaSyurui.UNKNOWN:
                //    case KomaSyurui.TOTTA_KOMA_NASI:
                //        break;
                //    default:
                //        sb.Append("(");
                //        sb.Append(Converter.SyuruiToSfen(process.Sengo,process.TottaKoma));
                //        sb.Append(")");
                //        break;
                //}

                sb.Append(" ");

            }

            //System.Console.WriteLine("size(" + fugoList.Count+")"+sb.ToString());

            return sb.ToString();
        }


        /// <summary>
        /// ************************************************************************************************************************
        /// 棋譜最後の駒の成／不成を切り替えます。
        /// ************************************************************************************************************************
        /// </summary>
        public static void Nari_KifuLast(Shape_BtnKoma btnKoma, Shape_PnlTaikyoku brush_PnlTaikyoku, Kifu kifu)
        {
            TeProcess last;
            kifu.PopLast(out last);


            if (null != last)
            {
                // 最後の符号の成／不成も変えます。

                if (kifu.Kyokumen.KomaDoors[btnKoma.Handle].Nari)
                {
                    //------------------------------
                    // 成→不成
                    //------------------------------

                    //----------
                    // 棋譜
                    //----------
                    TeProcess process = new TeProcess(
                        last.Sengo,
                        last.Okiba,
                        last.SrcOkiba,
                        last.Suji,
                        last.Dan,
                        last.SrcSuji,
                        last.SrcDan,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].ToFunariCase(),//不成
                        last.SrcSyurui,
                        last.TottaKoma
                        );
                    kifu.Add(process);

                    //----------
                    // 駒
                    //----------
                    kifu.Kyokumen.KomaDoors[btnKoma.Handle] = new TeProcess(
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].Sengo,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].Okiba,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].Suji,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].Dan,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].ToFunariCase()//不成
                        );

                }
                else
                {
                    //------------------------------
                    // 不成→成
                    //------------------------------

                    //----------
                    // 棋譜
                    //----------
                    TeProcess process = new TeProcess(
                        last.Sengo,
                        last.Okiba,
                        last.SrcOkiba,
                        last.Suji,
                        last.Dan,
                        last.SrcSuji,
                        last.SrcDan,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].ToNariCase(),//成り
                        last.SrcSyurui,
                        last.TottaKoma
                        );
                    kifu.Add(process);

                    //----------
                    // 駒
                    //----------
                    kifu.Kyokumen.KomaDoors[btnKoma.Handle] = new TeProcess(
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].Sengo,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].Okiba,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].Suji,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].Dan,
                        kifu.Kyokumen.KomaDoors[btnKoma.Handle].ToNariCase()//成り
                        );

                }
            }


            //----------
            // 駒の選択解除
            //----------
            btnKoma.Deselect();
        }

    }


}
