﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.Text.RegularExpressions;
using System.Diagnostics;
using System.Threading;

using Xenon.KifuNarabe.L00_DammyConsole;
using Xenon.KifuNarabe.L01_Communication;
using Xenon.KifuNarabe.L02_Common;
using Xenon.KifuNarabe.L03_TuginoItte;
using Xenon.KifuNarabe.L04_Shape;
using Xenon.KifuNarabe.L05_Server;
using Xenon.KifuNarabe.L06_Ui;

namespace Xenon.KifuNarabe.L06_Ui
{

    /// <summary>
    /// ************************************************************************************************************************
    /// メイン画面です。
    /// ************************************************************************************************************************
    /// </summary>
    [Serializable]
    public partial class Ui_PnlMain : UserControl
    {

        #region プロパティー類

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// グラフィックを描くツールは全部この中です。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Shape_PnlTaikyoku Brush_PnlTaikyoku
        {
            get
            {
                return this.brush_PnlTaikyoku;
            }
        }
        private Shape_PnlTaikyoku brush_PnlTaikyoku;

        /// <summary>
        /// ------------------------------------------------------------------------------------------------------------------------
        /// 将棋の状況は全部この中です。
        /// ------------------------------------------------------------------------------------------------------------------------
        /// </summary>
        public Kifu Kifu
        {
            get
            {
                return this.kifu;
            }
        }
        private Kifu kifu;

        #endregion


        #region ゲームエンジンの振りをするメソッド

        /// <summary>
        /// ************************************************************************************************************************
        /// 入力欄のテキストを取得します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <returns></returns>
        public string ReadLine1()
        {
            return this.txtInput1.Text;
        }

        private const int NSQUARE = 9 * 9;

        /// <summary>
        /// ************************************************************************************************************************
        /// 入力欄のテキストを取得します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <returns></returns>
        public string ReadLine2()
        {


            // 現在の局面は、これが駒の配列になっています。
            // this.Kifu.Kyokumen.KomaDoors[～]

            //------------------------------------------------------------
            // 表について
            //------------------------------------------------------------

            //
            // 配列の添え字は次の通り。
            //
            //    ９　８　７　６　５　４　３　２　１
            //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
            //  │ 0│ 1│ 2│ 3│ 4│ 5│ 6│ 7│ 8│一
            //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
            //  │ 9│10│11│12│13│14│15│16│17│二
            //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
            //  │18│19│20│21│22│23│24│25│26│三
            //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
            //  │27│28│29│30│31│32│33│34│35│四
            //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
            //　│36│37│38│39│40│41│42│43│44│五
            //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
            //  │45│46│47│48│49│50│51│52│53│六
            //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
            //  │54│55│56│57│58│59│60│61│62│七
            //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
            //  │63│64│65│66│67│68│69│70│71│八
            //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
            //  │72│73│74│75│76│77│78│79│80│九
            //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

            //------------------------------------------------------------
            // 先手駒について
            //------------------------------------------------------------

            // 先手駒の位置を表にします。
            bool[] wallSTable = new bool[Ui_PnlMain.NSQUARE];

            // 先手駒の利きを表にします。
            bool[] kikiSTable = new bool[Ui_PnlMain.NSQUARE];

            foreach (int hKoma in this.Kifu.Kyokumen.HKomas_ByOkibaSengo(Okiba.ShogiBan, Sengo.Sente))
            {
                TeProcess koma = this.Kifu.Kyokumen.KomaDoors[hKoma];

                // 壁
                wallSTable[(koma.Dan-1) * 9 + (9 - koma.Suji)] = true;

                // 利き
                kikiSTable[(koma.Dan-1) * 9 + (9 - koma.Suji)] = true;//FIXME:嘘
            }

            //------------------------------------------------------------
            // 後手駒について
            //------------------------------------------------------------

            // 先手駒の位置を表にします。
            bool[] wallGTable = new bool[Ui_PnlMain.NSQUARE];

            // 先手駒の利きを表にします。
            bool[] kikiGTable = new bool[Ui_PnlMain.NSQUARE];

            foreach (int hKoma in this.Kifu.Kyokumen.HKomas_ByOkibaSengo(Okiba.ShogiBan, Sengo.Gote))
            {
                TeProcess koma = this.Kifu.Kyokumen.KomaDoors[hKoma];

                // 壁
                wallGTable[(koma.Dan - 1) * 9 + (9 - koma.Suji)] = true;

                // 利き
                kikiGTable[(koma.Dan - 1) * 9 + (9 - koma.Suji)] = true;//FIXME:嘘
            }


            string tuginoItte = "▲９九王嘘";


            List<int> hKomas = this.Kifu.Kyokumen.HKomas_ByOkibaSengo(Okiba.ShogiBan, this.Kifu.Sengo);
            if (0<hKomas.Count)
            {
                int hKoma = hKomas[this.Kifu.Random.Next(hKomas.Count)];//ランダムに１つ。
                        
                TeProcess koma = this.Kifu.Kyokumen.KomaDoors[hKoma];

                TeProcess tuginoItteData;
                switch (this.Kifu.Sengo)
                {
                    case Sengo.Gote:
                        // 後手番です。
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        // 前に１つ突き出させます。
                        tuginoItteData = new TeProcess(
                                this.Kifu.Sengo,
                                Okiba.ShogiBan,
                                koma.Okiba,
                                koma.Suji,
                                koma.Dan + 1,
                                koma.Suji,
                                koma.Dan,
                                koma.Syurui,
                                koma.Syurui,
                                KomaSyurui.CTRL_NOTHING_PROPERTY
                                );
                        break;
                    default:
                        // 先手番です。
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        // 前に１つ突き出させます。
                        tuginoItteData = new TeProcess(
                                this.Kifu.Sengo,
                                Okiba.ShogiBan,
                                koma.Okiba,
                                koma.Suji,
                                koma.Dan - 1,
                                koma.Suji,
                                koma.Dan,
                                koma.Syurui,
                                koma.Syurui,
                                KomaSyurui.CTRL_NOTHING_PROPERTY
                                );
                        break;
                }

                Fugo fugo = TuginoItte_JapanFugo.GetText_FromData( tuginoItteData, this.Kifu.Kyokumen );
                tuginoItte = fugo.ToText(true, tuginoItteData, this.Kifu.Last);//FIXME
            }


            return tuginoItte;
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 出力欄（上段）にテキストを出力します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <returns></returns>
        public void WriteLine1(string text)
        {
            this.txtOutput1.Text = text;
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 出力欄（下段）にテキストを出力します。
        /// ************************************************************************************************************************
        /// </summary>
        /// <returns></returns>
        public void WriteLine2(string text)
        {
            this.txtOutput2.Text = text;
        }
        #endregion



        /// <summary>
        /// ************************************************************************************************************************
        /// コンストラクターです。
        /// ************************************************************************************************************************
        /// </summary>
        public Ui_PnlMain()
        {
            InitializeComponent();
        }


        private void timer1_Tick(object sender, EventArgs e)
        {
            //Application.DoEvents();

            //Thread.Sleep(40);

            //System.Console.WriteLine("ThreadTask99() input99=[" + input99 + "]");

            if (0 < Ui_PnlMain.input99.Length)
            {
                System.Console.WriteLine("input99=[" + input99 + "]");

                this.txtInput1.Text += input99;
                input99 = "";

                // 最善手を入力欄１に足します。
                RequestForMain requestForMain = new RequestForMain();
                requestForMain.RequestAppendInputTextString = DammyConsole.DefaultDammyConsole.ReadLine2();
                // 反映させます。
                this.Response(requestForMain);
                // コマ送りします。
                Ui_02Menu.Komaokuri(ref requestForMain, brush_PnlTaikyoku, kifu);
                // 反映させます。
                this.Response(requestForMain);
                //requestForMain = new RequestForMain();//クリアー


            }

        }
        //private static Thread thread99;
        public static string input99 = "";



        /// <summary>
        /// ************************************************************************************************************************
        /// 起動直後の流れです。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ui_PnlMain_Load(object sender, EventArgs e)
        {

            //----------
            // 監視
            //----------
            this.timer1.Start();
            //Ui_PnlMain.thread99 = new Thread(new ThreadStart(this.ThreadTask99));
            //Ui_PnlMain.thread99.IsBackground = true;
            //Ui_PnlMain.thread99.Start();

            //----------
            // 将棋エンジン（外部アプリ）の起動
            //----------
            //ShogiEngineService.StartShogiEngine();

            //----------
            // コンソールの振り
            //----------
            //
            //      このメインパネルに、コンソールの振りをさせます。
            //      将棋エンジンがあれば、将棋エンジンの入出力を返すように内部を改造してください。
            //
            DammyConsole.SetShogiEngine(this.ReadLine1, this.ReadLine2, this.WriteLine1, this.WriteLine2);


            //----------
            // ブラシ
            //----------
            //
            //      ボタンや将棋盤などを描画するツールを、事前準備しておきます。
            //
            this.brush_PnlTaikyoku = new Shape_PnlTaikyoku();


            //----------
            // 棋譜
            //----------
            //
            //      先後や駒など、対局に用いられる事柄、物を事前準備しておきます。
            //
            this.kifu = new Kifu();


            //----------
            // 駒の並べ方
            //----------
            //
            //      平手に並べます。
            //
            Ui_03Action.Syokihaichi_Hirate( this.Kifu);
            // 再描画
            foreach (Shape_BtnKoma btnKoma in brush_PnlTaikyoku.BtnKomaDoors)
            {
                Ui_03Action.Refresh_KomaLocation(btnKoma.Handle, brush_PnlTaikyoku, kifu.Kyokumen);
            }


            //----------
            // 画面の出力欄
            //----------
            //
            //      出力欄（上下段）を空っぽにします。
            //
            this.WriteLine1("");
            this.WriteLine2("");


            // これで、最初に見える画面の準備は終えました。
            // あとは、操作者の入力を待ちます。
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 描画するのはここです。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ui_PnlMain_Paint(object sender, PaintEventArgs e)
        {
            //------------------------------
            // 画面の描画です。
            //------------------------------
            this.Brush_PnlTaikyoku.Paint(sender, e, this.Kifu);
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// マウスが動いたときの挙動です。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ui_PnlMain_MouseMove(object sender, MouseEventArgs e)
        {
            // このメインパネルに、何かして欲しいという要求は、ここに入れられます。
            RequestForMain requestForMain = new RequestForMain();


            //------------------------------
            // マウスが重なったときの、表示物の反応や、将棋データの変更がこの中に書かれています。
            //------------------------------
            Ui_01EventDetail.Ui_PnlMain_MouseMoveDetail(ref requestForMain, e.Location, this.Brush_PnlTaikyoku);

            //------------------------------
            // このメインパネルの反応
            //------------------------------
            this.Response(requestForMain);
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// マウスのボタンを押下したときの挙動です。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ui_PnlMain_MouseDown(object sender, MouseEventArgs e)
        {
            // このメインパネルに、何かして欲しいという要求は、ここに入れられます。
            RequestForMain requestForMain = new RequestForMain();


            //------------------------------
            // マウスボタンが押下されたときの、表示物の反応や、将棋データの変更がこの中に書かれています。
            //------------------------------
            Ui_01EventDetail.Ui_PnlMain_MouseDownDetail(this, ref requestForMain, e, this.Brush_PnlTaikyoku, this.Kifu);


            //------------------------------
            // このメインパネルの反応
            //------------------------------
            this.Response(requestForMain);
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// マウスのボタンが放されたときの挙動です。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ui_PnlMain_MouseUp(object sender, MouseEventArgs e)
        {
            // このメインパネルに、何かして欲しいという要求は、ここに入れられます。
            RequestForMain requestForMain = new RequestForMain();

            //------------------------------
            // マウスボタンが放されたときの、表示物の反応や、将棋データの変更がこの中に書かれています。
            //------------------------------
            Ui_01EventDetail.Ui_PnlMain_MouseUpDetail(ref requestForMain, e, this.Brush_PnlTaikyoku, this.Kifu);

            //------------------------------
            // このメインパネルの反応
            //------------------------------
            this.Response(requestForMain);
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 入力欄の表示・出力欄の表示・再描画
        /// ************************************************************************************************************************
        /// 
        /// このメインパネルに何かして欲しいことがあれば、
        /// RequestForMain に要望を入れて、この関数を呼び出してください。
        ///
        /// 同時には処理できない項目もあります。
        /// </summary>
        /// <param name="requestForMain"></param>
        public void Response(RequestForMain requestForMain)
        {
            //------------------------------
            // 入力欄の表示
            //------------------------------
            if (requestForMain.CanInputTextFlag)
            {
                // 指定のテキストで上書きします。
                this.txtInput1.Text = requestForMain.RequestInputTextString;
            }
            else if (requestForMain.CanAppendInputTextFlag)
            {
                // 指定のテキストを後ろに足します。
                this.txtInput1.Text += requestForMain.RequestAppendInputTextString;
            }

            //------------------------------
            // 出力欄（上・下段）の表示
            //------------------------------
            if (requestForMain.RequestOutputHtml)
            {
                // 上段を、HTMLで上書きします。
                this.txtOutput1.Text = Ui_02Menu.CreateHtml(kifu.Kyokumen);
            }
            else if (requestForMain.RequestOutputKifu)
            {
                // 出力欄（上下段）に、棋譜を出力します。
                this.WriteLine1(Ui_02Menu.ToKifu1Text(this.Kifu));
                this.WriteLine2(Ui_02Menu.ToKifu2Text(this.Kifu));
            }
            else if (requestForMain.RequestClearTxtOutput)
            {
                // 出力欄（上下段）を空っぽにします。
                this.WriteLine1("");
                this.WriteLine2("");
            }

            //------------------------------
            // 再描画
            //------------------------------
            if (requestForMain.RequestRefresh)
            {
                this.Refresh();
            }
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 出力欄（上段）でキーボードのキーが押されたときの挙動をここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtOutput1_KeyDown(object sender, KeyEventArgs e)
        {
            //------------------------------
            // [Ctrl]+[A] で、全選択します。
            //------------------------------
            if (e.KeyCode == System.Windows.Forms.Keys.A & e.Control == true)
            {
                ((TextBox)sender).SelectAll();
            } 
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 出力欄（下段）でキーボードのキーが押されたときの挙動をここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtOutput2_KeyDown(object sender, KeyEventArgs e)
        {
            //------------------------------
            // [Ctrl]+[A] で、全選択します。
            //------------------------------
            if (e.KeyCode == System.Windows.Forms.Keys.A & e.Control == true)
            {
                ((TextBox)sender).SelectAll();
            } 
        }

        /// <summary>
        /// ************************************************************************************************************************
        /// 入力欄でキーボードのキーが押されたときの挙動をここに書きます。
        /// ************************************************************************************************************************
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtInput1_KeyDown(object sender, KeyEventArgs e)
        {
            //------------------------------
            // [Ctrl]+[A] で、全選択します。
            //------------------------------
            if (e.KeyCode == System.Windows.Forms.Keys.A & e.Control == true)
            {
                ((TextBox)sender).SelectAll();
            } 
        }
    }
}
